#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
module: docker_network
short_description: Manage Docker networks
description:
  - Create/remove Docker networks and connect containers to them.
  - Performs largely the same function as the C(docker network) CLI subcommand.

extends_documentation_fragment:
  - community.docker.docker.api_documentation
  - community.docker.attributes
  - community.docker.attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full

options:
  name:
    description:
      - Name of the network to operate on.
    type: str
    required: true
    aliases:
      - network_name

  connected:
    description:
      - List of container names or container IDs to connect to a network.
      - Please note that the module only makes sure that these containers are connected to the network,
        but does not care about connection options. If you rely on specific IP addresses etc., use the
        M(community.docker.docker_container) module to ensure your containers are correctly connected to this network.
    type: list
    elements: str
    default: []
    aliases:
      - containers

  driver:
    description:
      - Specify the type of network. Docker provides bridge and overlay drivers, but 3rd party drivers can also be used.
    type: str
    default: bridge

  driver_options:
    description:
      - Dictionary of network settings. Consult docker docs for valid options and values.
    type: dict
    default: {}

  force:
    description:
      - With state V(absent) forces disconnecting all containers from the
        network prior to deleting the network. With state V(present) will
        disconnect all containers, delete the network and re-create the
        network.
      - This option is required if you have changed the IPAM or driver options
        and want an existing network to be updated to use the new options.
    type: bool
    default: false

  appends:
    description:
      - By default the connected list is canonical, meaning containers not on the list are removed from the network.
      - Use O(appends) to leave existing containers connected.
    type: bool
    default: false
    aliases:
      - incremental

  enable_ipv6:
    description:
      - Enable IPv6 networking.
    type: bool

  ipam_driver:
    description:
      - Specify an IPAM driver.
    type: str

  ipam_driver_options:
    description:
      - Dictionary of IPAM driver options.
    type: dict

  ipam_config:
    description:
      - List of IPAM config blocks. Consult
        L(Docker docs,https://docs.docker.com/compose/compose-file/compose-file-v2/#ipam) for valid options and values.
        Note that O(ipam_config[].iprange) is spelled differently here (we use the notation from the Docker SDK for Python).
    type: list
    elements: dict
    suboptions:
      subnet:
        description:
          - IP subset in CIDR notation.
        type: str
      iprange:
        description:
          - IP address range in CIDR notation.
        type: str
      gateway:
        description:
          - IP gateway address.
        type: str
      aux_addresses:
        description:
          - Auxiliary IP addresses used by Network driver, as a mapping from hostname to IP.
        type: dict

  state:
    description:
      - V(absent) deletes the network. If a network has connected containers, it
        cannot be deleted. Use the O(force) option to disconnect all containers
        and delete the network.
      - V(present) creates the network, if it does not already exist with the
        specified parameters, and connects the list of containers provided via
        the connected parameter. Containers not on the list will be disconnected.
        An empty list will leave no containers connected to the network. Use the
        O(appends) option to leave existing containers connected. Use the O(force)
        options to force re-creation of the network.
    type: str
    default: present
    choices:
      - absent
      - present

  internal:
    description:
      - Restrict external access to the network.
    type: bool

  labels:
    description:
      - Dictionary of labels.
    type: dict
    default: {}

  scope:
    description:
      - Specify the network's scope.
    type: str
    choices:
      - local
      - global
      - swarm

  attachable:
    description:
      - If enabled, and the network is in the global scope, non-service containers on worker nodes will be able to connect to the network.
    type: bool

notes:
  - When network options are changed, the module disconnects all containers from the network, deletes the network, and re-creates the network.
    It does not try to reconnect containers, except the ones listed in (O(connected), and even for these, it does not consider specific
    connection options like fixed IP addresses or MAC addresses. If you need more control over how the containers are connected to the
    network, loop the M(community.docker.docker_container) module to loop over your containers to make sure they are connected properly.
  - The module does not support Docker Swarm. This means that it will not try to disconnect or reconnect services. If services are connected to the
    network, deleting the network will fail. When network options are changed, the network has to be deleted and recreated, so this will
    fail as well.

author:
  - "Ben Keith (@keitwb)"
  - "Chris Houseknecht (@chouseknecht)"
  - "Dave Bendit (@DBendit)"

requirements:
  - "Docker API >= 1.25"
'''

EXAMPLES = '''
- name: Create a network
  community.docker.docker_network:
    name: network_one

- name: Remove all but selected list of containers
  community.docker.docker_network:
    name: network_one
    connected:
      - container_a
      - container_b
      - container_c

- name: Remove a single container
  community.docker.docker_network:
    name: network_one
    connected: "{{ fulllist|difference(['container_a']) }}"

- name: Add a container to a network, leaving existing containers connected
  community.docker.docker_network:
    name: network_one
    connected:
      - container_a
    appends: true

- name: Create a network with driver options
  community.docker.docker_network:
    name: network_two
    driver_options:
      com.docker.network.bridge.name: net2

- name: Create a network with custom IPAM config
  community.docker.docker_network:
    name: network_three
    ipam_config:
      - subnet: 172.23.27.0/24
        gateway: 172.23.27.2
        iprange: 172.23.27.0/26
        aux_addresses:
          host1: 172.23.27.3
          host2: 172.23.27.4

- name: Create a network with labels
  community.docker.docker_network:
    name: network_four
    labels:
      key1: value1
      key2: value2

- name: Create a network with IPv6 IPAM config
  community.docker.docker_network:
    name: network_ipv6_one
    enable_ipv6: true
    ipam_config:
      - subnet: fdd1:ac8c:0557:7ce1::/64

- name: Create a network with IPv6 and custom IPv4 IPAM config
  community.docker.docker_network:
    name: network_ipv6_two
    enable_ipv6: true
    ipam_config:
      - subnet: 172.24.27.0/24
      - subnet: fdd1:ac8c:0557:7ce2::/64

- name: Delete a network, disconnecting all containers
  community.docker.docker_network:
    name: network_one
    state: absent
    force: true
'''

RETURN = '''
network:
    description:
    - Network inspection results for the affected network.
    returned: success
    type: dict
    sample: {}
'''

import re
import traceback

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.docker.plugins.module_utils.common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils.util import (
    DockerBaseClass,
    DifferenceTracker,
    clean_dict_booleans_for_docker_api,
)
from ansible_collections.community.docker.plugins.module_utils._api.errors import DockerException


class TaskParameters(DockerBaseClass):
    def __init__(self, client):
        super(TaskParameters, self).__init__()
        self.client = client

        self.name = None
        self.connected = None
        self.driver = None
        self.driver_options = None
        self.ipam_driver = None
        self.ipam_driver_options = None
        self.ipam_config = None
        self.appends = None
        self.force = None
        self.internal = None
        self.labels = None
        self.debug = None
        self.enable_ipv6 = None
        self.scope = None
        self.attachable = None

        for key, value in client.module.params.items():
            setattr(self, key, value)


def container_names_in_network(network):
    return [c['Name'] for c in network['Containers'].values()] if network['Containers'] else []


CIDR_IPV4 = re.compile(r'^([0-9]{1,3}\.){3}[0-9]{1,3}/([0-9]|[1-2][0-9]|3[0-2])$')
CIDR_IPV6 = re.compile(r'^[0-9a-fA-F:]+/([0-9]|[1-9][0-9]|1[0-2][0-9])$')


def validate_cidr(cidr):
    """Validate CIDR. Return IP version of a CIDR string on success.

    :param cidr: Valid CIDR
    :type cidr: str
    :return: ``ipv4`` or ``ipv6``
    :rtype: str
    :raises ValueError: If ``cidr`` is not a valid CIDR
    """
    if CIDR_IPV4.match(cidr):
        return 'ipv4'
    elif CIDR_IPV6.match(cidr):
        return 'ipv6'
    raise ValueError('"{0}" is not a valid CIDR'.format(cidr))


def normalize_ipam_config_key(key):
    """Normalizes IPAM config keys returned by Docker API to match Ansible keys.

    :param key: Docker API key
    :type key: str
    :return Ansible module key
    :rtype str
    """
    special_cases = {
        'AuxiliaryAddresses': 'aux_addresses'
    }
    return special_cases.get(key, key.lower())


def dicts_are_essentially_equal(a, b):
    """Make sure that a is a subset of b, where None entries of a are ignored."""
    for k, v in a.items():
        if v is None:
            continue
        if b.get(k) != v:
            return False
    return True


class DockerNetworkManager(object):

    def __init__(self, client):
        self.client = client
        self.parameters = TaskParameters(client)
        self.check_mode = self.client.check_mode
        self.results = {
            u'changed': False,
            u'actions': []
        }
        self.diff = self.client.module._diff
        self.diff_tracker = DifferenceTracker()
        self.diff_result = dict()

        self.existing_network = self.get_existing_network()

        if not self.parameters.connected and self.existing_network:
            self.parameters.connected = container_names_in_network(self.existing_network)

        if self.parameters.ipam_config:
            try:
                for ipam_config in self.parameters.ipam_config:
                    validate_cidr(ipam_config['subnet'])
            except ValueError as e:
                self.client.fail(to_native(e))

        if self.parameters.driver_options:
            self.parameters.driver_options = clean_dict_booleans_for_docker_api(self.parameters.driver_options)

        state = self.parameters.state
        if state == 'present':
            self.present()
        elif state == 'absent':
            self.absent()

        if self.diff or self.check_mode or self.parameters.debug:
            if self.diff:
                self.diff_result['before'], self.diff_result['after'] = self.diff_tracker.get_before_after()
            self.results['diff'] = self.diff_result

    def get_existing_network(self):
        return self.client.get_network(name=self.parameters.name)

    def has_different_config(self, net):
        '''
        Evaluates an existing network and returns a tuple containing a boolean
        indicating if the configuration is different and a list of differences.

        :param net: the inspection output for an existing network
        :return: (bool, list)
        '''
        differences = DifferenceTracker()
        if self.parameters.driver and self.parameters.driver != net['Driver']:
            differences.add('driver',
                            parameter=self.parameters.driver,
                            active=net['Driver'])
        if self.parameters.driver_options:
            if not net.get('Options'):
                differences.add('driver_options',
                                parameter=self.parameters.driver_options,
                                active=net.get('Options'))
            else:
                for key, value in self.parameters.driver_options.items():
                    if not (key in net['Options']) or value != net['Options'][key]:
                        differences.add('driver_options.%s' % key,
                                        parameter=value,
                                        active=net['Options'].get(key))

        if self.parameters.ipam_driver:
            if not net.get('IPAM') or net['IPAM']['Driver'] != self.parameters.ipam_driver:
                differences.add('ipam_driver',
                                parameter=self.parameters.ipam_driver,
                                active=net.get('IPAM'))

        if self.parameters.ipam_driver_options is not None:
            ipam_driver_options = net['IPAM'].get('Options') or {}
            if ipam_driver_options != self.parameters.ipam_driver_options:
                differences.add('ipam_driver_options',
                                parameter=self.parameters.ipam_driver_options,
                                active=ipam_driver_options)

        if self.parameters.ipam_config is not None and self.parameters.ipam_config:
            if not net.get('IPAM') or not net['IPAM']['Config']:
                differences.add('ipam_config',
                                parameter=self.parameters.ipam_config,
                                active=net.get('IPAM', {}).get('Config'))
            else:
                # Put network's IPAM config into the same format as module's IPAM config
                net_ipam_configs = []
                for net_ipam_config in net['IPAM']['Config']:
                    config = dict()
                    for k, v in net_ipam_config.items():
                        config[normalize_ipam_config_key(k)] = v
                    net_ipam_configs.append(config)
                # Compare lists of dicts as sets of dicts
                for idx, ipam_config in enumerate(self.parameters.ipam_config):
                    net_config = dict()
                    for net_ipam_config in net_ipam_configs:
                        if dicts_are_essentially_equal(ipam_config, net_ipam_config):
                            net_config = net_ipam_config
                            break
                    for key, value in ipam_config.items():
                        if value is None:
                            # due to recursive argument_spec, all keys are always present
                            # (but have default value None if not specified)
                            continue
                        if value != net_config.get(key):
                            differences.add('ipam_config[%s].%s' % (idx, key),
                                            parameter=value,
                                            active=net_config.get(key))

        if self.parameters.enable_ipv6 is not None and self.parameters.enable_ipv6 != net.get('EnableIPv6', False):
            differences.add('enable_ipv6',
                            parameter=self.parameters.enable_ipv6,
                            active=net.get('EnableIPv6', False))

        if self.parameters.internal is not None and self.parameters.internal != net.get('Internal', False):
            differences.add('internal',
                            parameter=self.parameters.internal,
                            active=net.get('Internal'))

        if self.parameters.scope is not None and self.parameters.scope != net.get('Scope'):
            differences.add('scope',
                            parameter=self.parameters.scope,
                            active=net.get('Scope'))

        if self.parameters.attachable is not None and self.parameters.attachable != net.get('Attachable', False):
            differences.add('attachable',
                            parameter=self.parameters.attachable,
                            active=net.get('Attachable'))
        if self.parameters.labels:
            if not net.get('Labels'):
                differences.add('labels',
                                parameter=self.parameters.labels,
                                active=net.get('Labels'))
            else:
                for key, value in self.parameters.labels.items():
                    if not (key in net['Labels']) or value != net['Labels'][key]:
                        differences.add('labels.%s' % key,
                                        parameter=value,
                                        active=net['Labels'].get(key))

        return not differences.empty, differences

    def create_network(self):
        if not self.existing_network:
            data = {
                'Name': self.parameters.name,
                'Driver': self.parameters.driver,
                'Options': self.parameters.driver_options,
                'IPAM': None,
                'CheckDuplicate': None,
            }

            if self.parameters.enable_ipv6:
                data['EnableIPv6'] = True
            if self.parameters.internal:
                data['Internal'] = True
            if self.parameters.scope is not None:
                data['Scope'] = self.parameters.scope
            if self.parameters.attachable is not None:
                data['Attachable'] = self.parameters.attachable
            if self.parameters.labels is not None:
                data["Labels"] = self.parameters.labels

            ipam_pools = []
            if self.parameters.ipam_config:
                for ipam_pool in self.parameters.ipam_config:
                    ipam_pools.append({
                        'Subnet': ipam_pool['subnet'],
                        'IPRange': ipam_pool['iprange'],
                        'Gateway': ipam_pool['gateway'],
                        'AuxiliaryAddresses': ipam_pool['aux_addresses'],
                    })

            if self.parameters.ipam_driver or self.parameters.ipam_driver_options or ipam_pools:
                # Only add IPAM if a driver was specified or if IPAM parameters were
                # specified. Leaving this parameter out can significantly speed up
                # creation; on my machine creation with this option needs ~15 seconds,
                # and without just a few seconds.
                data['IPAM'] = {
                    'Driver': self.parameters.ipam_driver,
                    'Config': ipam_pools or [],
                    'Options': self.parameters.ipam_driver_options,
                }

            if not self.check_mode:
                resp = self.client.post_json_to_json('/networks/create', data=data)
                self.client.report_warnings(resp, ['Warning'])
                self.existing_network = self.client.get_network(network_id=resp['Id'])
            self.results['actions'].append("Created network %s with driver %s" % (self.parameters.name, self.parameters.driver))
            self.results['changed'] = True

    def remove_network(self):
        if self.existing_network:
            self.disconnect_all_containers()
            if not self.check_mode:
                self.client.delete_call('/networks/{0}', self.parameters.name)
            self.results['actions'].append("Removed network %s" % (self.parameters.name,))
            self.results['changed'] = True

    def is_container_connected(self, container_name):
        if not self.existing_network:
            return False
        return container_name in container_names_in_network(self.existing_network)

    def connect_containers(self):
        for name in self.parameters.connected:
            if not self.is_container_connected(name):
                if not self.check_mode:
                    data = {
                        "Container": name,
                        "EndpointConfig": None,
                    }
                    self.client.post_json('/networks/{0}/connect', self.parameters.name, data=data)
                self.results['actions'].append("Connected container %s" % (name,))
                self.results['changed'] = True
                self.diff_tracker.add('connected.{0}'.format(name), parameter=True, active=False)

    def disconnect_missing(self):
        if not self.existing_network:
            return
        containers = self.existing_network['Containers']
        if not containers:
            return
        for c in containers.values():
            name = c['Name']
            if name not in self.parameters.connected:
                self.disconnect_container(name)

    def disconnect_all_containers(self):
        containers = self.client.get_network(name=self.parameters.name)['Containers']
        if not containers:
            return
        for cont in containers.values():
            self.disconnect_container(cont['Name'])

    def disconnect_container(self, container_name):
        if not self.check_mode:
            data = {"Container": container_name}
            self.client.post_json('/networks/{0}/disconnect', self.parameters.name, data=data)
        self.results['actions'].append("Disconnected container %s" % (container_name,))
        self.results['changed'] = True
        self.diff_tracker.add('connected.{0}'.format(container_name),
                              parameter=False,
                              active=True)

    def present(self):
        different = False
        differences = DifferenceTracker()
        if self.existing_network:
            different, differences = self.has_different_config(self.existing_network)

        self.diff_tracker.add('exists', parameter=True, active=self.existing_network is not None)
        if self.parameters.force or different:
            self.remove_network()
            self.existing_network = None

        self.create_network()
        self.connect_containers()
        if not self.parameters.appends:
            self.disconnect_missing()

        if self.diff or self.check_mode or self.parameters.debug:
            self.diff_result['differences'] = differences.get_legacy_docker_diffs()
            self.diff_tracker.merge(differences)

        if not self.check_mode and not self.parameters.debug:
            self.results.pop('actions')

        network_facts = self.get_existing_network()
        self.results['network'] = network_facts

    def absent(self):
        self.diff_tracker.add('exists', parameter=False, active=self.existing_network is not None)
        self.remove_network()


def main():
    argument_spec = dict(
        name=dict(type='str', required=True, aliases=['network_name']),
        connected=dict(type='list', default=[], elements='str', aliases=['containers']),
        state=dict(type='str', default='present', choices=['present', 'absent']),
        driver=dict(type='str', default='bridge'),
        driver_options=dict(type='dict', default={}),
        force=dict(type='bool', default=False),
        appends=dict(type='bool', default=False, aliases=['incremental']),
        ipam_driver=dict(type='str'),
        ipam_driver_options=dict(type='dict'),
        ipam_config=dict(type='list', elements='dict', options=dict(
            subnet=dict(type='str'),
            iprange=dict(type='str'),
            gateway=dict(type='str'),
            aux_addresses=dict(type='dict'),
        )),
        enable_ipv6=dict(type='bool'),
        internal=dict(type='bool'),
        labels=dict(type='dict', default={}),
        debug=dict(type='bool', default=False),
        scope=dict(type='str', choices=['local', 'global', 'swarm']),
        attachable=dict(type='bool'),
    )

    option_minimal_versions = dict(
        scope=dict(docker_api_version='1.30'),
        attachable=dict(docker_api_version='1.26'),
    )

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        # "The docker server >= 1.10.0"
        option_minimal_versions=option_minimal_versions,
    )

    try:
        cm = DockerNetworkManager(client)
        client.module.exit_json(**cm.results)
    except DockerException as e:
        client.fail('An unexpected Docker error occurred: {0}'.format(to_native(e)), exception=traceback.format_exc())
    except RequestException as e:
        client.fail(
            'An unexpected requests error occurred when trying to talk to the Docker daemon: {0}'.format(to_native(e)),
            exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type
import re

from ansible.module_utils.common.network import to_netmask
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import to_list

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.cli.config.bgp.address_family import (
    AddressFamily,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.cli.config.bgp.neighbors import (
    Neighbors,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.providers.providers import (
    CliProvider,
    register_provider,
)


REDISTRIBUTE_PROTOCOLS = [
    "ospf",
    "ospfv3",
    "eigrp",
    "isis",
    "static",
    "connected",
    "odr",
    "lisp",
    "mobile",
    "rip",
]


@register_provider("ios", "ios_bgp")
class Provider(CliProvider):
    def render(self, config=None):
        commands = list()

        existing_as = None
        if config:
            match = re.search(r"router bgp (\d+)", config, re.M)
            if match:
                existing_as = match.group(1)

        operation = self.params["operation"]

        context = None
        if self.params["config"]:
            context = "router bgp %s" % self.get_value("config.bgp_as")

        if operation == "delete":
            if existing_as:
                commands.append("no router bgp %s" % existing_as)
            elif context:
                commands.append("no %s" % context)

        else:
            self._validate_input(config)
            if operation == "replace":
                if existing_as and int(existing_as) != self.get_value("config.bgp_as"):
                    commands.append("no router bgp %s" % existing_as)
                    config = None

            elif operation == "override":
                if existing_as:
                    commands.append("no router bgp %s" % existing_as)
                config = None

            context_commands = list()

            for key, value in iteritems(self.get_value("config")):
                if value is not None:
                    meth = getattr(self, "_render_%s" % key, None)
                    if meth:
                        resp = meth(config)
                        if resp:
                            context_commands.extend(to_list(resp))

            if context and context_commands:
                commands.append(context)
                commands.extend(context_commands)
                commands.append("exit")
        return commands

    def _render_router_id(self, config=None):
        cmd = "bgp router-id %s" % self.get_value("config.router_id")
        if not config or cmd not in config:
            return cmd

    def _render_log_neighbor_changes(self, config=None):
        cmd = "bgp log-neighbor-changes"
        log_neighbor_changes = self.get_value("config.log_neighbor_changes")
        if log_neighbor_changes is True:
            if not config or cmd not in config:
                return cmd
        elif log_neighbor_changes is False:
            if config and cmd in config:
                return "no %s" % cmd

    def _render_networks(self, config=None):
        commands = list()
        safe_list = list()

        for entry in self.get_value("config.networks"):
            network = entry["prefix"]
            cmd = "network %s" % network
            if entry["masklen"] and entry["masklen"] not in (24, 16, 8):
                cmd += " mask %s" % to_netmask(entry["masklen"])
                network += " mask %s" % to_netmask(entry["masklen"])

            if entry["route_map"]:
                cmd += " route-map %s" % entry["route_map"]
                network += " route-map %s" % entry["route_map"]

            safe_list.append(network)

            if not config or cmd not in config:
                commands.append(cmd)

        if self.params["operation"] == "replace":
            if config:
                matches = re.findall(r"network (.*)", config, re.M)
                for entry in set(matches).difference(safe_list):
                    commands.append("no network %s" % entry)

        return commands

    def _render_neighbors(self, config):
        """generate bgp neighbor configuration"""
        return Neighbors(self.params).render(config)

    def _render_address_family(self, config):
        """generate address-family configuration"""
        return AddressFamily(self.params).render(config)

    def _validate_input(self, config=None):
        def device_has_AF(config):
            return re.search(r"address-family (?:.*)", config)

        address_family = self.get_value("config.address_family")
        root_networks = self.get_value("config.networks")
        operation = self.params["operation"]

        if operation == "replace":
            if address_family and root_networks:
                for item in address_family:
                    if item["networks"]:
                        raise ValueError(
                            "operation is replace but provided both root level network(s) and network(s) under %s %s address family"
                            % (item["afi"], item["safi"]),
                        )

            if root_networks and config and device_has_AF(config):
                raise ValueError(
                    "operation is replace and device has one or more address family activated but root level network(s) provided",
                )



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_lsm_cluster
short_description: Manages lsm-cluster objects on Checkpoint over Web Services API
description:
  - Manages lsm-cluster objects on Checkpoint devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
version_added: "2.3.0"
author: "Shiran Golzar (@chkp-shirango)"
options:
  main_ip_address:
    description:
      - Main IP address.
    type: str
  name_prefix:
    description:
      - A prefix added to the profile name and creates the LSM cluster name.
    type: str
  name_suffix:
    description:
      - A suffix added to the profile name and creates the LSM cluster name.
    type: str
  security_profile:
    description:
      - LSM profile.
    type: str
    required: True
  interfaces:
    description:
      - Interfaces.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Interface name.
        type: str
      ip_address_override:
        description:
          - IP address override. Net mask is defined by the attached LSM profile.
        type: str
      member_network_override:
        description:
          - Member network override. Net mask is defined by the attached LSM profile.
        type: str
  members:
    description:
      - Members.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Object name.
        type: str
      provisioning_settings:
        description:
          - Provisioning settings. This field is relevant just for SMB clusters.
        type: dict
        suboptions:
          provisioning_profile:
            description:
              - Provisioning profile.
            type: str
      provisioning_state:
        description:
          - Provisioning state. This field is relevant just for SMB clusters. By default the state is 'manual'- enable provisioning but not attach
            to profile.If 'using-profile' state is provided a provisioning profile must be provided in provisioning-settings.
        type: str
        choices: ['off', 'manual', 'using-profile']
      sic:
        description:
          - Secure Internal Communication.
        type: dict
        suboptions:
          ip_address:
            description:
              - IP address. When IP address is provided- initiate trusted communication immediately using this IP address.
            type: str
          one_time_password:
            description:
              - One-time password. When one-time password is provided without ip-address- trusted communication is
                automatically initiated when the gateway connects to the Security Management server for the first time.
            type: str
      tags:
        description:
          - Collection of tag identifiers.
        type: list
        elements: str
      color:
        description:
          - Color of the object. Should be one of existing colors.
        type: str
        choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange',
                 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray',
                 'light green', 'lemon chiffon', 'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive',
                 'orange', 'red', 'sienna', 'yellow']
      comments:
        description:
          - Comments string.
        type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-lsm-cluster
  cp_mgmt_lsm_cluster:
    interfaces:
    - ip_address_override: 192.168.8.197
      member_network_override: 192.168.8.0
      name: eth0
      new_name: WAN
    - ip_address_override: 10.8.197.1
      member_network_override: 10.8.197.0
      name: eth1
      new_name: LAN1
    - member_network_override: 10.10.10.0
      name: eth2
    main_ip_address: 192.168.8.197
    members:
    - name: Gaia_gw1
      sic:
        ip_address: 192.168.8.200
        one_time_password: aaaa
    - name: Gaia_gw2
      sic:
        ip_address: 192.168.8.202
        one_time_password: aaaa
    name_prefix: Gaia_
    security_profile: gaia_cluster
    state: present

- name: set-lsm-cluster
  cp_mgmt_lsm_cluster:
    interfaces:
    - ip_address_override: 192.168.8.197
      member_network_override: 192.168.8.0
      name: eth0
      new_name: WAN
    - ip_address_override: 10.8.197.1
      member_network_override: 10.8.197.0
      name: eth1
      new_name: LAN1
    - member_network_override: 10.10.10.0
      name: eth2
    members:
    - name: Gaia_gw1
      sic:
        ip_address: 192.168.8.200
        one_time_password: aaaa
    - name: Gaia_gw2
      sic:
        ip_address: 192.168.8.202
        one_time_password: aaaa
    name: Gaia_gaia_cluster
    state: present

- name: delete-lsm-cluster
  cp_mgmt_lsm_cluster:
    name: lsm_cluster
    state: absent
"""

RETURN = """
cp_mgmt_lsm_cluster:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        main_ip_address=dict(type="str"),
        name_prefix=dict(type="str"),
        name_suffix=dict(type="str"),
        security_profile=dict(type="str", required=True),
        interfaces=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str"),
                ip_address_override=dict(type="str"),
                member_network_override=dict(type="str"),
            ),
        ),
        members=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str"),
                provisioning_settings=dict(
                    type="dict",
                    options=dict(provisioning_profile=dict(type="str")),
                ),
                provisioning_state=dict(
                    type="str", choices=["off", "manual", "using-profile"]
                ),
                sic=dict(
                    type="dict",
                    options=dict(
                        ip_address=dict(type="str"),
                        one_time_password=dict(type="str", no_log=True),
                    ),
                ),
                tags=dict(type="list", elements="str"),
                color=dict(
                    type="str",
                    choices=[
                        "aquamarine",
                        "black",
                        "blue",
                        "crete blue",
                        "burlywood",
                        "cyan",
                        "dark green",
                        "khaki",
                        "orchid",
                        "dark orange",
                        "dark sea green",
                        "pink",
                        "turquoise",
                        "dark blue",
                        "firebrick",
                        "brown",
                        "forest green",
                        "gold",
                        "dark gold",
                        "gray",
                        "dark gray",
                        "light green",
                        "lemon chiffon",
                        "coral",
                        "sea green",
                        "sky blue",
                        "magenta",
                        "purple",
                        "slate blue",
                        "violet red",
                        "navy blue",
                        "olive",
                        "orange",
                        "red",
                        "sienna",
                        "yellow",
                    ],
                ),
                comments=dict(type="str"),
            ),
        ),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    # Create lsm-cluster name
    name = module.params["security_profile"]

    if module.params["name_prefix"]:
        name = module.params["name_prefix"] + name
    if module.params["name_suffix"]:
        name = name + module.params["name_suffix"]
    module.params["name"] = name

    api_call_object = "lsm-cluster"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2019, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    DOCUMENTATION = '''
options:
  api_url:
    description:
      - cloudscale.ch API URL.
      - This can also be passed in the C(CLOUDSCALE_API_URL) environment variable.
    default: https://api.cloudscale.ch/v1
    type: str
    version_added: 1.3.0
  api_token:
    description:
      - cloudscale.ch API token.
      - This can also be passed in the C(CLOUDSCALE_API_TOKEN) environment variable.
    required: true
    type: str
  api_timeout:
    description:
      - Timeout in seconds for calls to the cloudscale.ch API.
      - This can also be passed in the C(CLOUDSCALE_API_TIMEOUT) environment variable.
    default: 45
    type: int
notes:
  - All operations are performed using the cloudscale.ch public API v1.
  - "For details consult the full API documentation: U(https://www.cloudscale.ch/en/api/v1)."
  - A valid API token is required for all operations. You can create as many tokens as you like using the cloudscale.ch control panel at
    U(https://control.cloudscale.ch).
'''



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos_lldp_global class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    remove_empties,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts


class Lldp_global(ConfigBase):
    """
    The nxos_lldp_global class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["lldp_global"]

    def __init__(self, module):
        super(Lldp_global, self).__init__(module)

    def get_lldp_global_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        lldp_global_facts = facts["ansible_network_resources"].get("lldp_global")
        if not lldp_global_facts:
            return {}
        return lldp_global_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = list()
        warnings = list()

        if self.state in self.ACTION_STATES:
            existing_lldp_global_facts = self.get_lldp_global_facts()
        else:
            existing_lldp_global_facts = {}

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_lldp_global_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_lldp_global_facts = self.get_lldp_global_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_lldp_global_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_lldp_global_facts
            if result["changed"]:
                result["after"] = changed_lldp_global_facts

        elif self.state == "gathered":
            result["gathered"] = changed_lldp_global_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_lldp_global_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_lldp_global_facts
        resp = self.set_state(remove_empties(want), have)
        return resp

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(state),
            )
        commands = list()

        if state == "deleted":
            commands = self._state_deleted(have)
        elif state in ["merged", "rendered"]:
            commands = self._state_merged(want, have)
        elif state in ["replaced", "overridden"]:
            commands = self._state_replaced(want, have)
        return commands

    def _state_replaced(self, want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        merge_dict = dict_diff(have, want)
        #  merge_dict will contain new and unique values in want
        delete_dict = self.find_delete_params(have, want)
        self._module.params["state"] = "deleted"
        commands.extend(self._state_deleted(delete_dict))  # delete
        self._module.params["state"] = "merged"
        commands.extend(self.set_commands(merge_dict))  # merge
        self._module.params["state"] = "replaced"
        return commands

    def delete_nested_dict(self, have, want):
        """
        Returns tlv_select nested dict that needs to be defaulted
        """
        outer_dict = {}
        for key, val in have.items():
            inner_dict = {}
            if not isinstance(val, dict):
                if key not in want.keys():
                    inner_dict.update({key: val})
                    return inner_dict
            else:
                if key in want.keys():
                    outer_dict.update({key: self.delete_nested_dict(val, want[key])})
                else:
                    outer_dict.update({key: val})
        return outer_dict

    def find_delete_params(self, have, want):
        """
        Returns parameters that are present in have and not in want, that need to be defaulted
        """
        delete_dict = {}
        for key, val in have.items():
            if key not in want.keys():
                delete_dict.update({key: val})
            else:
                if key == "tlv_select":
                    delete_dict.update(
                        {key: self.delete_nested_dict(have["tlv_select"], want["tlv_select"])},
                    )
        return delete_dict

    def _state_merged(self, want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        diff = dict_diff(have, want)
        commands.extend(self.set_commands(diff))
        return commands

    def _state_deleted(self, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        if have:
            for key, val in have.items():
                if "tlv_select" in key:
                    commands.extend(self.process_nested_dict(val))
                else:
                    if key == "port_id":
                        key = "portid-subtype"
                    commands.append("no lldp " + key + " " + str(val))

        return commands

    def set_commands(self, diff):
        commands = []
        for key, val in diff.items():
            commands.extend(self.add_commands(key, val))
        return commands

    def add_commands(self, key, val):
        command = []
        if "port_id" in key:
            command.append("lldp portid-subtype " + str(val))
        elif "tlv_select" in key:
            command.extend(self.process_nested_dict(val))
        else:
            if val:
                command.append("lldp " + key + " " + str(val))
        return command

    def process_nested_dict(self, val):
        nested_commands = []
        for k, v in val.items():
            if isinstance(v, dict):
                for k1, v1 in v.items():
                    com1 = "lldp tlv-select "
                    com2 = ""
                    if "system" in k:
                        com2 = "system-" + k1
                    elif "management_address" in k:
                        com2 = "management-address " + k1
                    elif "port" in k:
                        com2 = "port-" + k1

                    com1 += com2
                    com1 = self.negate_command(com1, v1)
                    nested_commands.append(com1)
            else:
                com1 = "lldp tlv-select "
                if "power_management" in k:
                    com1 += "power-management"
                else:
                    com1 += k

                com1 = self.negate_command(com1, v)
                nested_commands.append(com1)
        return nested_commands

    def negate_command(self, command, val):
        # for merged, replaced vals need to be checked to add 'no'
        if self._module.params["state"] == "merged":
            if not val:
                command = "no " + command
        return command



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or
# https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


DOCUMENTATION = """
module: nxos_zone_zoneset
short_description: Configuration of zone/zoneset for Cisco NXOS MDS Switches.
description:
- Configuration of zone/zoneset for Cisco MDS NXOS.
version_added: 1.0.0
author:
- Suhas Bharadwaj (@srbharadwaj) (subharad@cisco.com)
notes:
- Tested against Cisco MDS NX-OS 8.4(1)
options:
  zone_zoneset_details:
    description:
    - List of zone/zoneset details to be added or removed
    type: list
    elements: dict
    suboptions:
      vsan:
        description:
        - vsan id
        required: true
        type: int
      mode:
        description:
        - mode of the zone for the vsan
        choices:
        - enhanced
        - basic
        type: str
      default_zone:
        description:
        - default zone behaviour for the vsan
        choices:
        - permit
        - deny
        type: str
      smart_zoning:
        description:
        - Removes the vsan if True
        type: bool
      zone:
        description:
        - List of zone options for that vsan
        type: list
        elements: dict
        suboptions:
          name:
            description:
            - name of the zone
            required: true
            type: str
          remove:
            description:
            - Deletes the zone if True
            type: bool
            default: false
          members:
            description:
            - Members of the zone that needs to be removed or added
            type: list
            elements: dict
            suboptions:
              pwwn:
                description:
                - pwwn member of the zone, use alias 'device_alias' as option for
                  device_alias member
                aliases:
                - device_alias
                required: true
                type: str
              remove:
                description:
                - Removes member from the zone if True
                type: bool
                default: false
              devtype:
                description:
                - devtype of the zone member used along with Smart zoning config
                choices:
                - initiator
                - target
                - both
                type: str
      zoneset:
        description:
        - List of zoneset options for the vsan
        type: list
        elements: dict
        suboptions:
          name:
            description:
            - name of the zoneset
            required: true
            type: str
          remove:
            description:
            - Removes zoneset if True
            type: bool
            default: false
          action:
            description:
            - activates/de-activates the zoneset
            choices:
            - activate
            - deactivate
            type: str
          members:
            description:
            - Members of the zoneset that needs to be removed or added
            type: list
            elements: dict
            suboptions:
              name:
                description:
                - name of the zone that needs to be added to the zoneset or removed
                  from the zoneset
                required: true
                type: str
              remove:
                description:
                - Removes zone member from the zoneset
                type: bool
                default: false
"""

EXAMPLES = """
- name: Test that zone/zoneset module works
  cisco.nxos.nxos_zone_zoneset:
    zone_zoneset_details:
    - mode: enhanced
      vsan: 22
      zone:
      - members:
        - pwwn: 11:11:11:11:11:11:11:11
        - device_alias: test123
        - pwwn: 61:61:62:62:12:12:12:12
          remove: true
        name: zoneA
      - members:
        - pwwn: 10:11:11:11:11:11:11:11
        - pwwn: 62:62:62:62:21:21:21:21
        name: zoneB
      - name: zoneC
        remove: true
      zoneset:
      - action: activate
        members:
        - name: zoneA
        - name: zoneB
        - name: zoneC
          remove: true
        name: zsetname1
      - action: deactivate
        name: zsetTestExtra
        remove: true
    - mode: basic
      smart_zoning: true
      vsan: 21
      zone:
      - members:
        - devtype: both
          pwwn: 11:11:11:11:11:11:11:11
        - pwwn: 62:62:62:62:12:12:12:12
        - devtype: both
          pwwn: 92:62:62:62:12:12:1a:1a
          remove: true
        name: zone21A
      - members:
        - pwwn: 10:11:11:11:11:11:11:11
        - pwwn: 62:62:62:62:21:21:21:21
        name: zone21B
      zoneset:
      - action: activate
        members:
        - name: zone21A
        - name: zone21B
        name: zsetname212

"""

RETURN = """
commands:
  description: commands sent to the device
  returned: always
  type: list
  sample:
    - terminal dont-ask
    - zone name zoneA vsan 923
    - member pwwn 11:11:11:11:11:11:11:11
    - no member device-alias test123
    - zone commit vsan 923
    - no terminal dont-ask
messages:
  description: debug messages
  returned: always
  type: list
  sample:
    - "zone mode is already enhanced ,no change in zone mode configuration for vsan 922"
    - "zone member '11:11:11:11:11:11:11:11' is already present in zone 'zoneA' in vsan 922 hence nothing to add"
    - "zone member 'test123' is already present in zone 'zoneA' in vsan 922 hence nothing to add"
    - "zone member '61:61:62:62:12:12:12:12' is not present in zone 'zoneA' in vsan 922 hence nothing to remove"
    - "zone member '10:11:11:11:11:11:11:11' is already present in zone 'zoneB' in vsan 922 hence nothing to add"
    - "zone member '62:62:62:62:21:21:21:21' is already present in zone 'zoneB' in vsan 922 hence nothing to add"
    - "zone 'zoneC' is not present in vsan 922 , so nothing to remove"
"""


import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


__metaclass__ = type


class ShowZonesetActive(object):
    """docstring for ShowZonesetActive"""

    def __init__(self, module, vsan):
        self.vsan = vsan
        self.module = module
        self.activeZSName = None
        self.parseCmdOutput()

    def execute_show_zoneset_active_cmd(self):
        command = "show zoneset active vsan " + str(self.vsan) + " | grep zoneset"
        output = execute_show_command(command, self.module)[0]
        return output

    def parseCmdOutput(self):
        patZoneset = r"zoneset name (\S+) vsan " + str(self.vsan)
        output = self.execute_show_zoneset_active_cmd().split("\n")
        if len(output) == 0:
            return
        else:
            for line in output:
                line = line.strip()
                mzs = re.match(patZoneset, line.strip())
                if mzs:
                    self.activeZSName = mzs.group(1).strip()
                    return

    def isZonesetActive(self, zsname):
        if zsname == self.activeZSName:
            return True
        return False


class ShowZoneset(object):
    """docstring for ShowZoneset"""

    def __init__(self, module, vsan):
        self.vsan = vsan
        self.module = module
        self.zsDetails = {}
        self.parseCmdOutput()

    def execute_show_zoneset_cmd(self):
        command = "show zoneset vsan " + str(self.vsan)
        output = execute_show_command(command, self.module)[0]
        return output

    def parseCmdOutput(self):
        patZoneset = r"zoneset name (\S+) vsan " + str(self.vsan)
        patZone = r"zone name (\S+) vsan " + str(self.vsan)
        output = self.execute_show_zoneset_cmd().split("\n")
        for line in output:
            line = line.strip()
            mzs = re.match(patZoneset, line.strip())
            mz = re.match(patZone, line.strip())
            if mzs:
                zonesetname = mzs.group(1).strip()
                self.zsDetails[zonesetname] = []
                continue
            elif mz:
                zonename = mz.group(1).strip()
                v = self.zsDetails[zonesetname]
                v.append(zonename)
                self.zsDetails[zonesetname] = v

    def isZonesetPresent(self, zsname):
        return zsname in self.zsDetails.keys()

    def isZonePresentInZoneset(self, zsname, zname):
        if zsname in self.zsDetails.keys():
            return zname in self.zsDetails[zsname]
        return False


class ShowZone(object):
    """docstring for ShowZone"""

    def __init__(self, module, vsan):
        self.vsan = vsan
        self.module = module
        self.zDetails = {}
        self.parseCmdOutput()

    def execute_show_zone_vsan_cmd(self):
        command = "show zone vsan " + str(self.vsan)
        output = execute_show_command(command, self.module)[0]
        return output

    def parseCmdOutput(self):
        patZone = r"zone name (\S+) vsan " + str(self.vsan)
        output = self.execute_show_zone_vsan_cmd().split("\n")
        for line in output:
            line = re.sub(r"[\[].*?[\]]", "", line)
            line = " ".join(line.strip().split())
            if "init" in line:
                line = line.replace("init", "initiator")
            m = re.match(patZone, line)
            if m:
                zonename = m.group(1).strip()
                self.zDetails[zonename] = []
                continue
            else:
                # For now we support only pwwn and device-alias under zone
                # Ideally should use 'supported_choices'....but maybe next
                # time.
                if "pwwn" in line or "device-alias" in line:
                    v = self.zDetails[zonename]
                    v.append(line)
                    self.zDetails[zonename] = v

    def isZonePresent(self, zname):
        return zname in self.zDetails.keys()

    def isZoneMemberPresent(self, zname, cmd):
        if zname in self.zDetails.keys():
            zonememlist = self.zDetails[zname]
            for eachline in zonememlist:
                if cmd in eachline:
                    return True
        return False

    def get_zDetails(self):
        return self.zDetails


class ShowZoneStatus(object):
    """docstring for ShowZoneStatus"""

    def __init__(self, module, vsan):
        self.vsan = vsan
        self.vsanAbsent = False
        self.module = module
        self.default_zone = ""
        self.mode = ""
        self.session = ""
        self.sz = ""
        self.locked = False
        self.update()

    def execute_show_zone_status_cmd(self):
        command = "show zone status vsan " + str(self.vsan)
        output = execute_show_command(command, self.module)[0]
        return output

    def update(self):
        output = self.execute_show_zone_status_cmd().split("\n")

        patfordefzone = "VSAN: " + str(self.vsan) + r" default-zone:\s+(\S+).*"
        patformode = r".*mode:\s+(\S+).*"
        patforsession = r"^session:\s+(\S+).*"
        patforsz = r".*smart-zoning:\s+(\S+).*"
        for line in output:
            if "is not configured" in line:
                self.vsanAbsent = True
                break
            mdefz = re.match(patfordefzone, line.strip())
            mmode = re.match(patformode, line.strip())
            msession = re.match(patforsession, line.strip())
            msz = re.match(patforsz, line.strip())

            if mdefz:
                self.default_zone = mdefz.group(1)
            if mmode:
                self.mode = mmode.group(1)
            if msession:
                self.session = msession.group(1)
                if self.session != "none":
                    self.locked = True
            if msz:
                self.sz = msz.group(1)

    def isLocked(self):
        return self.locked

    def getDefaultZone(self):
        return self.default_zone

    def getMode(self):
        return self.mode

    def getSmartZoningStatus(self):
        return self.sz

    def isVsanAbsent(self):
        return self.vsanAbsent


def execute_show_command(command, module, command_type="cli_show"):
    output = "text"
    commands = [{"command": command, "output": output}]
    return run_commands(module, commands)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def getMemType(supported_choices, allmemkeys, default="pwwn"):
    for eachchoice in supported_choices:
        if eachchoice in allmemkeys:
            return eachchoice
    return default


def main():
    supported_choices = ["device_alias"]
    zone_member_spec = dict(
        pwwn=dict(required=True, type="str", aliases=["device_alias"]),
        devtype=dict(type="str", choices=["initiator", "target", "both"]),
        remove=dict(type="bool", default=False),
    )

    zone_spec = dict(
        name=dict(required=True, type="str"),
        members=dict(type="list", elements="dict", options=zone_member_spec),
        remove=dict(type="bool", default=False),
    )

    zoneset_member_spec = dict(
        name=dict(required=True, type="str"),
        remove=dict(type="bool", default=False),
    )

    zoneset_spec = dict(
        name=dict(type="str", required=True),
        members=dict(type="list", elements="dict", options=zoneset_member_spec),
        remove=dict(type="bool", default=False),
        action=dict(type="str", choices=["activate", "deactivate"]),
    )

    zonedetails_spec = dict(
        vsan=dict(required=True, type="int"),
        mode=dict(type="str", choices=["enhanced", "basic"]),
        default_zone=dict(type="str", choices=["permit", "deny"]),
        smart_zoning=dict(type="bool"),
        zone=dict(type="list", elements="dict", options=zone_spec),
        zoneset=dict(type="list", elements="dict", options=zoneset_spec),
    )

    argument_spec = dict(
        zone_zoneset_details=dict(type="list", elements="dict", options=zonedetails_spec),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    messages = list()
    commands = list()
    result = {"changed": False}

    commands_executed = []
    listOfZoneDetails = module.params["zone_zoneset_details"]
    for eachZoneZonesetDetail in listOfZoneDetails:
        vsan = eachZoneZonesetDetail["vsan"]
        op_mode = eachZoneZonesetDetail["mode"]
        op_default_zone = eachZoneZonesetDetail["default_zone"]
        op_smart_zoning = eachZoneZonesetDetail["smart_zoning"]
        op_zone = eachZoneZonesetDetail["zone"]
        op_zoneset = eachZoneZonesetDetail["zoneset"]

        # Step1: execute show zone status and get
        shZoneStatusObj = ShowZoneStatus(module, vsan)
        sw_default_zone = shZoneStatusObj.getDefaultZone()
        sw_mode = shZoneStatusObj.getMode()
        sw_smart_zoning = shZoneStatusObj.getSmartZoningStatus()

        if sw_smart_zoning.lower() == "Enabled".lower():
            sw_smart_zoning_bool = True
        else:
            sw_smart_zoning_bool = False

        if shZoneStatusObj.isVsanAbsent():
            module.fail_json(
                msg="Vsan " + str(vsan) + " is not present in the switch. Hence cannot procced.",
            )

        if shZoneStatusObj.isLocked():
            module.fail_json(
                msg="zone has acquired lock on the switch for vsan "
                + str(vsan)
                + ". Hence cannot procced.",
            )

        # Process zone default zone options
        if op_default_zone is not None:
            if op_default_zone != sw_default_zone:
                if op_default_zone == "permit":
                    commands_executed.append("zone default-zone permit vsan " + str(vsan))
                    messages.append(
                        "default zone configuration changed from deny to permit for vsan "
                        + str(vsan),
                    )
                else:
                    commands_executed.append("no zone default-zone permit vsan " + str(vsan))
                    messages.append(
                        "default zone configuration changed from permit to deny for vsan "
                        + str(vsan),
                    )
            else:
                messages.append(
                    "default zone is already "
                    + op_default_zone
                    + " ,no change in default zone configuration for vsan "
                    + str(vsan),
                )

        # Process zone mode options
        if op_mode is not None:
            if op_mode != sw_mode:
                if op_mode == "enhanced":
                    commands_executed.append("zone mode enhanced vsan " + str(vsan))
                    messages.append(
                        "zone mode configuration changed from basic to enhanced for vsan "
                        + str(vsan),
                    )
                else:
                    commands_executed.append("no zone mode enhanced vsan " + str(vsan))
                    messages.append(
                        "zone mode configuration changed from enhanced to basic for vsan "
                        + str(vsan),
                    )
            else:
                messages.append(
                    "zone mode is already "
                    + op_mode
                    + " ,no change in zone mode configuration for vsan "
                    + str(vsan),
                )

        # Process zone smart-zone options
        if op_smart_zoning is not None:
            if op_smart_zoning != sw_smart_zoning_bool:
                if op_smart_zoning:
                    commands_executed.append("zone smart-zoning enable vsan " + str(vsan))
                    messages.append("smart-zoning enabled for vsan " + str(vsan))
                else:
                    commands_executed.append("no zone smart-zoning enable vsan " + str(vsan))
                    messages.append("smart-zoning disabled for vsan " + str(vsan))
            else:
                messages.append(
                    "smart-zoning is already set to "
                    + sw_smart_zoning
                    + " , no change in smart-zoning configuration for vsan "
                    + str(vsan),
                )

        # Process zone member options
        # TODO: Obviously this needs to be cleaned up properly, as there are a lot of ifelse statements which is bad
        # Will take it up later becoz of time constraints
        if op_zone is not None:
            shZoneObj = ShowZone(module, vsan)
            for eachzone in op_zone:
                zname = eachzone["name"]
                zmembers = eachzone["members"]
                removeflag = eachzone["remove"]
                if removeflag:
                    if shZoneObj.isZonePresent(zname):
                        messages.append("zone '" + zname + "' is removed from vsan " + str(vsan))
                        commands_executed.append("no zone name " + zname + " vsan " + str(vsan))
                    else:
                        messages.append(
                            "zone '"
                            + zname
                            + "' is not present in vsan "
                            + str(vsan)
                            + " , so nothing to remove",
                        )
                else:
                    if zmembers is None:
                        if shZoneObj.isZonePresent(zname):
                            messages.append(
                                "zone '" + zname + "' is already present in vsan " + str(vsan),
                            )
                        else:
                            commands_executed.append("zone name " + zname + " vsan " + str(vsan))
                            messages.append("zone '" + zname + "' is created in vsan " + str(vsan))
                    else:
                        cmdmemlist = []
                        for eachmem in zmembers:
                            memtype = getMemType(supported_choices, eachmem.keys())
                            cmd = memtype.replace("_", "-") + " " + eachmem[memtype]
                            if op_smart_zoning or sw_smart_zoning_bool:
                                if eachmem["devtype"] is not None:
                                    cmd = cmd + " " + eachmem["devtype"]
                            if eachmem["remove"]:
                                if shZoneObj.isZonePresent(zname):
                                    if shZoneObj.isZoneMemberPresent(zname, cmd):
                                        cmd = "no member " + cmd
                                        cmdmemlist.append(cmd)
                                        if op_smart_zoning and eachmem["devtype"] is not None:
                                            messages.append(
                                                "removing zone member '"
                                                + eachmem[memtype]
                                                + " of device type '"
                                                + eachmem["devtype"]
                                                + "' from zone '"
                                                + zname
                                                + "' in vsan "
                                                + str(vsan),
                                            )
                                        else:
                                            messages.append(
                                                "removing zone member '"
                                                + eachmem[memtype]
                                                + "' from zone '"
                                                + zname
                                                + "' in vsan "
                                                + str(vsan),
                                            )
                                    else:
                                        if op_smart_zoning and eachmem["devtype"] is not None:
                                            messages.append(
                                                "zone member '"
                                                + eachmem[memtype]
                                                + "' of device type '"
                                                + eachmem["devtype"]
                                                + "' is not present in zone '"
                                                + zname
                                                + "' in vsan "
                                                + str(vsan)
                                                + " hence nothing to remove",
                                            )
                                        else:
                                            messages.append(
                                                "zone member '"
                                                + eachmem[memtype]
                                                + "' is not present in zone '"
                                                + zname
                                                + "' in vsan "
                                                + str(vsan)
                                                + " hence nothing to remove",
                                            )
                                else:
                                    messages.append(
                                        "zone '"
                                        + zname
                                        + "' is not present in vsan "
                                        + str(vsan)
                                        + " , hence cannot remove the members",
                                    )

                            else:
                                if shZoneObj.isZoneMemberPresent(zname, cmd):
                                    if op_smart_zoning and eachmem["devtype"] is not None:
                                        messages.append(
                                            "zone member '"
                                            + eachmem[memtype]
                                            + "' of device type '"
                                            + eachmem["devtype"]
                                            + "' is already present in zone '"
                                            + zname
                                            + "' in vsan "
                                            + str(vsan)
                                            + " hence nothing to add",
                                        )
                                    else:
                                        messages.append(
                                            "zone member '"
                                            + eachmem[memtype]
                                            + "' is already present in zone '"
                                            + zname
                                            + "' in vsan "
                                            + str(vsan)
                                            + " hence nothing to add",
                                        )
                                else:
                                    cmd = "member " + cmd
                                    cmdmemlist.append(cmd)
                                    if op_smart_zoning and eachmem["devtype"] is not None:
                                        messages.append(
                                            "adding zone member '"
                                            + eachmem[memtype]
                                            + "' of device type '"
                                            + eachmem["devtype"]
                                            + "' to zone '"
                                            + zname
                                            + "' in vsan "
                                            + str(vsan),
                                        )
                                    else:
                                        messages.append(
                                            "adding zone member '"
                                            + eachmem[memtype]
                                            + "' to zone '"
                                            + zname
                                            + "' in vsan "
                                            + str(vsan),
                                        )
                        if len(cmdmemlist) != 0:
                            commands_executed.append("zone name " + zname + " vsan " + str(vsan))
                            commands_executed = commands_executed + cmdmemlist

        # Process zoneset member options
        if op_zoneset is not None:
            dactcmd = []
            actcmd = []
            shZonesetObj = ShowZoneset(module, vsan)
            shZonesetActiveObj = ShowZonesetActive(module, vsan)
            for eachzoneset in op_zoneset:
                zsetname = eachzoneset["name"]
                zsetmembers = eachzoneset["members"]
                removeflag = eachzoneset["remove"]
                actionflag = eachzoneset["action"]
                if removeflag:
                    if shZonesetObj.isZonesetPresent(zsetname):
                        messages.append(
                            "zoneset '" + zsetname + "' is removed from vsan " + str(vsan),
                        )
                        commands_executed.append(
                            "no zoneset name " + zsetname + " vsan " + str(vsan),
                        )
                    else:
                        messages.append(
                            "zoneset '"
                            + zsetname
                            + "' is not present in vsan "
                            + str(vsan)
                            + " ,hence there is nothing to remove",
                        )
                else:
                    if zsetmembers is not None:
                        cmdmemlist = []
                        for eachzsmem in zsetmembers:
                            zsetmem_name = eachzsmem["name"]
                            zsetmem_removeflag = eachzsmem["remove"]
                            if zsetmem_removeflag:
                                if shZonesetObj.isZonePresentInZoneset(zsetname, zsetmem_name):
                                    cmd = "no member " + zsetmem_name
                                    cmdmemlist.append(cmd)
                                    messages.append(
                                        "removing zoneset member '"
                                        + zsetmem_name
                                        + "' from zoneset '"
                                        + zsetname
                                        + "' in vsan "
                                        + str(vsan),
                                    )
                                else:
                                    messages.append(
                                        "zoneset member '"
                                        + zsetmem_name
                                        + "' is not present in zoneset '"
                                        + zsetname
                                        + "' in vsan "
                                        + str(vsan)
                                        + " ,hence there is nothing to remove",
                                    )
                            else:
                                if shZonesetObj.isZonePresentInZoneset(zsetname, zsetmem_name):
                                    messages.append(
                                        "zoneset member '"
                                        + zsetmem_name
                                        + "' is already present in zoneset '"
                                        + zsetname
                                        + "' in vsan "
                                        + str(vsan)
                                        + " ,hence there is nothing to add",
                                    )
                                else:
                                    cmd = "member " + zsetmem_name
                                    cmdmemlist.append(cmd)
                                    messages.append(
                                        "adding zoneset member '"
                                        + zsetmem_name
                                        + "' to zoneset '"
                                        + zsetname
                                        + "' in vsan "
                                        + str(vsan),
                                    )
                        if len(cmdmemlist) != 0:
                            commands_executed.append(
                                "zoneset name " + zsetname + " vsan " + str(vsan),
                            )
                            commands_executed = commands_executed + cmdmemlist
                    else:
                        if shZonesetObj.isZonesetPresent(zsetname):
                            messages.append(
                                "zoneset '"
                                + zsetname
                                + "' is already present in vsan "
                                + str(vsan),
                            )
                        else:
                            commands_executed.append(
                                "zoneset name " + zsetname + " vsan " + str(vsan),
                            )
                            messages.append(
                                "zoneset '" + zsetname + "' is created in vsan " + str(vsan),
                            )

                # Process zoneset activate options
                if actionflag == "deactivate":
                    if shZonesetActiveObj.isZonesetActive(zsetname):
                        messages.append(
                            "deactivating zoneset '" + zsetname + "' in vsan " + str(vsan),
                        )
                        dactcmd.append(
                            "no zoneset activate name " + zsetname + " vsan " + str(vsan),
                        )
                    else:
                        messages.append(
                            "zoneset '"
                            + zsetname
                            + "' in vsan "
                            + str(vsan)
                            + " is not activated, hence cannot deactivate",
                        )
                elif actionflag == "activate":
                    if commands_executed:
                        messages.append(
                            "activating zoneset '" + zsetname + "' in vsan " + str(vsan),
                        )
                        actcmd.append("zoneset activate name " + zsetname + " vsan " + str(vsan))
                    else:
                        messages.append(
                            "no changes to existing zoneset '"
                            + zsetname
                            + "' in vsan "
                            + str(vsan)
                            + " hence activate action is ignored",
                        )
            commands_executed = commands_executed + dactcmd + actcmd

        if commands_executed:
            if op_mode == "enhanced":
                commands_executed.append("zone commit vsan " + str(vsan))
            elif op_mode is None:
                if sw_mode == "enhanced":
                    commands_executed.append("zone commit vsan " + str(vsan))

    if commands_executed:
        commands_executed = ["terminal dont-ask"] + commands_executed + ["no terminal dont-ask"]

    cmds = flatten_list(commands_executed)
    if cmds:
        if module.check_mode:
            module.exit_json(
                changed=False,
                commands=cmds,
                msg="Check Mode: No cmds issued to the hosts",
            )
        else:
            result["changed"] = True
            commands = commands + cmds
            load_config(module, cmds)

    result["messages"] = messages
    result["commands"] = commands_executed
    result["warnings"] = warnings
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: docker_image_load

short_description: Load docker image(s) from archives

version_added: 1.3.0

description:
  - Load one or multiple Docker images from a C(.tar) archive, and return information on
    the loaded image(s).

extends_documentation_fragment:
  - community.docker.docker.api_documentation
  - community.docker.attributes
  - community.docker.attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  path:
    description:
      - The path to the C(.tar) archive to load Docker image(s) from.
    type: path
    required: true

requirements:
  - "Docker API >= 1.25"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_export
  - module: community.docker.docker_image_push
  - module: community.docker.docker_image_remove
  - module: community.docker.docker_image_tag
'''

EXAMPLES = '''
- name: Load all image(s) from the given tar file
  community.docker.docker_image_load:
    path: /path/to/images.tar
  register: result

- name: Print the loaded image names
  ansible.builtin.debug:
    msg: "Loaded the following images: {{ result.image_names | join(', ') }}"
'''

RETURN = '''
image_names:
    description: List of image names and IDs loaded from the archive.
    returned: success
    type: list
    elements: str
    sample:
      - 'hello-world:latest'
      - 'sha256:e004c2cc521c95383aebb1fb5893719aa7a8eae2e7a71f316a4410784edb00a9'
images:
    description: Image inspection results for the loaded images.
    returned: success
    type: list
    elements: dict
    sample: []
'''

import errno
import traceback

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.docker.plugins.module_utils.common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils.util import (
    DockerBaseClass,
    is_image_name_id,
)

from ansible_collections.community.docker.plugins.module_utils._api.errors import DockerException


class ImageManager(DockerBaseClass):
    def __init__(self, client, results):
        super(ImageManager, self).__init__()

        self.client = client
        self.results = results
        parameters = self.client.module.params
        self.check_mode = self.client.check_mode

        self.path = parameters['path']

        self.load_images()

    @staticmethod
    def _extract_output_line(line, output):
        '''
        Extract text line from stream output and, if found, adds it to output.
        '''
        if 'stream' in line or 'status' in line:
            # Make sure we have a string (assuming that line['stream'] and
            # line['status'] are either not defined, falsish, or a string)
            text_line = line.get('stream') or line.get('status') or ''
            output.extend(text_line.splitlines())

    def load_images(self):
        '''
        Load images from a .tar archive
        '''
        # Load image(s) from file
        load_output = []
        try:
            self.log("Opening image {0}".format(self.path))
            with open(self.path, 'rb') as image_tar:
                self.log("Loading images from {0}".format(self.path))
                res = self.client._post(self.client._url("/images/load"), data=image_tar, stream=True)
                for line in self.client._stream_helper(res, decode=True):
                    self.log(line, pretty_print=True)
                    self._extract_output_line(line, load_output)
        except EnvironmentError as exc:
            if exc.errno == errno.ENOENT:
                self.client.fail("Error opening archive {0} - {1}".format(self.path, to_native(exc)))
            self.client.fail("Error loading archive {0} - {1}".format(self.path, to_native(exc)), stdout='\n'.join(load_output))
        except Exception as exc:
            self.client.fail("Error loading archive {0} - {1}".format(self.path, to_native(exc)), stdout='\n'.join(load_output))

        # Collect loaded images
        loaded_images = []
        for line in load_output:
            if line.startswith('Loaded image:'):
                loaded_images.append(line[len('Loaded image:'):].strip())
            if line.startswith('Loaded image ID:'):
                loaded_images.append(line[len('Loaded image ID:'):].strip())

        if not loaded_images:
            self.client.fail("Detected no loaded images. Archive potentially corrupt?", stdout='\n'.join(load_output))

        images = []
        for image_name in loaded_images:
            if is_image_name_id(image_name):
                images.append(self.client.find_image_by_id(image_name))
            elif ':' in image_name:
                image_name, tag = image_name.rsplit(':', 1)
                images.append(self.client.find_image(image_name, tag))
            else:
                self.client.module.warn('Image name "{0}" is neither ID nor has a tag'.format(image_name))

        self.results['image_names'] = loaded_images
        self.results['images'] = images
        self.results['changed'] = True
        self.results['stdout'] = '\n'.join(load_output)


def main():
    client = AnsibleDockerClient(
        argument_spec=dict(
            path=dict(type='path', required=True),
        ),
        supports_check_mode=False,
    )

    try:
        results = dict(
            image_names=[],
            images=[],
        )

        ImageManager(client, results)
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail('An unexpected Docker error occurred: {0}'.format(to_native(e)), exception=traceback.format_exc())
    except RequestException as e:
        client.fail(
            'An unexpected requests error occurred when trying to talk to the Docker daemon: {0}'.format(to_native(e)),
            exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_udld_interface
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages UDLD interface configuration params.
description:
- Manages UDLD interface configuration params.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- Feature UDLD must be enabled on the device to use this module.
options:
  mode:
    description:
    - Manages UDLD mode for an interface.
    required: true
    choices:
    - enabled
    - disabled
    - aggressive
    type: str
  interface:
    description:
    - FULL name of the interface, i.e. Ethernet1/1-
    required: true
    type: str
  state:
    description:
    - Manage the state of the resource.
    required: false
    default: present
    choices:
    - present
    - absent
    type: str
"""
EXAMPLES = """
# ensure Ethernet1/1 is configured to be in aggressive mode
- cisco.nxos.nxos_udld_interface:
    interface: Ethernet1/1
    mode: aggressive
    state: present
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'

# Remove the aggressive config only if it's currently in aggressive mode and then disable udld (switch default)
- cisco.nxos.nxos_udld_interface:
    interface: Ethernet1/1
    mode: aggressive
    state: absent
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'

# ensure Ethernet1/1 has aggressive mode enabled
- cisco.nxos.nxos_udld_interface:
    interface: Ethernet1/1
    mode: enabled
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'
"""

RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"mode": "enabled"}
existing:
    description:
        - k/v pairs of existing configuration
    returned: always
    type: dict
    sample: {"mode": "aggressive"}
end_state:
    description: k/v pairs of configuration after module execution
    returned: always
    type: dict
    sample: {"mode": "enabled"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["interface ethernet1/33",
            "no udld aggressive ; no udld disable"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_udld_interface(module, interface):
    command = "show run udld all | section " + interface.title() + "$"
    interface_udld = {}
    mode = None
    mode_str = None
    try:
        body = run_commands(module, [{"command": command, "output": "text"}])[0]
        if "aggressive" in body:
            mode = "aggressive"
            mode_str = "aggressive"
        elif "no udld enable" in body:
            mode = "disabled"
            mode_str = "no udld enable"
        elif "no udld disable" in body:
            mode = "enabled"
            mode_str = "no udld disable"
        elif "udld disable" in body:
            mode = "disabled"
            mode_str = "udld disable"
        elif "udld enable" in body:
            mode = "enabled"
            mode_str = "udld enable"
        interface_udld["mode"] = mode

    except (KeyError, AttributeError, IndexError):
        interface_udld = {}

    return interface_udld, mode_str


def get_commands_config_udld_interface1(delta, interface, module, existing):
    commands = []
    mode = delta["mode"]
    if mode == "aggressive":
        commands.append("udld aggressive")
    else:
        commands.append("no udld aggressive")
    commands.insert(0, "interface {0}".format(interface))

    return commands


def get_commands_config_udld_interface2(delta, interface, module, existing):
    commands = []
    existing, mode_str = get_udld_interface(module, interface)
    mode = delta["mode"]
    if mode == "enabled":
        if mode_str == "no udld enable":
            command = "udld enable"
        else:
            command = "no udld disable"
    else:
        if mode_str == "no udld disable":
            command = "udld disable"
        else:
            command = "no udld enable"
    if command:
        commands.append(command)
        commands.insert(0, "interface {0}".format(interface))

    return commands


def get_commands_remove_udld_interface(delta, interface, module, existing):
    commands = []
    existing, mode_str = get_udld_interface(module, interface)

    mode = delta["mode"]
    if mode == "aggressive":
        command = "no udld aggressive"
    else:
        if mode == "enabled":
            if mode_str == "udld enable":
                command = "no udld enable"
            else:
                command = "udld disable"
        elif mode == "disabled":
            if mode_str == "no udld disable":
                command = "udld disable"
            else:
                command = "no udld enable"
    if command:
        commands.append(command)
        commands.insert(0, "interface {0}".format(interface))

    return commands


def main():
    argument_spec = dict(
        mode=dict(choices=["enabled", "disabled", "aggressive"], required=True),
        interface=dict(type="str", required=True),
        state=dict(choices=["absent", "present"], default="present"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()

    interface = module.params["interface"].lower()
    mode = module.params["mode"]
    state = module.params["state"]

    proposed = dict(mode=mode)
    existing, mode_str = get_udld_interface(module, interface)
    end_state = existing

    delta = dict(set(proposed.items()).difference(existing.items()))

    changed = False
    commands = []
    cmds = []
    if state == "present":
        if delta:
            command = get_commands_config_udld_interface1(delta, interface, module, existing)
            commands.append(command)
            cmds = flatten_list(commands)
            if module.check_mode:
                module.exit_json(changed=True, commands=cmds)
            else:
                changed = True
                load_config(module, cmds)

            if delta["mode"] == "enabled" or delta["mode"] == "disabled":
                commands = []
                command = get_commands_config_udld_interface2(delta, interface, module, existing)
                commands.append(command)
                cmds = flatten_list(commands)
                if module.check_mode:
                    module.exit_json(changed=True, commands=cmds)
                else:
                    load_config(module, cmds)

    else:
        common = set(proposed.items()).intersection(existing.items())
        if common:
            command = get_commands_remove_udld_interface(dict(common), interface, module, existing)
            cmds = flatten_list(commands)
            if module.check_mode:
                module.exit_json(changed=True, commands=cmds)
            else:
                changed = True
                load_config(module, cmds)

    if not module.check_mode:
        end_state, mode_str = get_udld_interface(module, interface)
        if "configure" in cmds:
            cmds.pop(0)

    results = {}
    results["proposed"] = proposed
    results["existing"] = existing
    results["end_state"] = end_state
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The asa_acls fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from copy import deepcopy

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)

from ansible_collections.cisco.asa.plugins.module_utils.network.asa.argspec.acls.acls import (
    AclsArgs,
)
from ansible_collections.cisco.asa.plugins.module_utils.network.asa.rm_templates.acls import (
    AclsTemplate,
)


class AclsFacts(object):
    """The asa_acls fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = AclsArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_acls_config(self, connection):
        return connection.get("sh access-list")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for ACLs
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_acls_config(connection)

        rmmod = NetworkTemplate(lines=data.splitlines(), tmplt=AclsTemplate())
        current = rmmod.parse()
        acls = list()
        if current.get("acls"):
            for key, val in iteritems(current.get("acls")):
                if val.get("name") == "cached":
                    continue
                for each in val.get("aces"):
                    if "protocol_number" in each:
                        each["protocol_options"] = {
                            "protocol_number": each["protocol_number"],
                        }
                        del each["protocol_number"]
                    if "icmp_icmp6_protocol" in each and each.get("protocol"):
                        each["protocol_options"] = {
                            each.get("protocol"): {
                                each["icmp_icmp6_protocol"].replace(
                                    "-",
                                    "_",
                                ): True,
                            },
                        }
                        del each["icmp_icmp6_protocol"]
                    elif (
                        each.get("protocol")
                        and each.get("protocol") != "icmp"
                        and each.get("protocol") != "icmp6"
                    ):
                        each["protocol_options"] = {each.get("protocol"): True}
                acls.append(val)
        facts = {}
        params = {}
        if acls:
            params = utils.validate_config(
                self.argument_spec,
                {"config": {"acls": acls}},
            )
            params = utils.remove_empties(params)
            facts["acls"] = params["config"]

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_inventory_onboarding_cloud_monitoring_imports
short_description: Resource module for organizations _inventory _onboarding _cloudmonitoring _imports
description:
- Manage operation create of the resource organizations _inventory _onboarding _cloudmonitoring _imports.
- Commits the import operation to complete the onboarding of a device into Dashboard for monitoring.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  devices:
    description: A set of device imports to commit.
    elements: dict
    suboptions:
      deviceId:
        description: Import ID from the Import operation.
        type: str
      networkId:
        description: Network Id.
        type: str
      udi:
        description: Device UDI certificate.
        type: str
    type: list
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for organizations createOrganizationInventoryOnboardingCloudMonitoringImport
  description: Complete reference of the createOrganizationInventoryOnboardingCloudMonitoringImport API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-organization-inventory-onboarding-cloud-monitoring-import
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_inventory_onboarding_cloud_monitoring_import,

  - Paths used are
    post /organizations/{organizationId}/inventory/onboarding/cloudMonitoring/imports,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_inventory_onboarding_cloud_monitoring_imports:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    devices:
    - deviceId: 161b2602-a713-4aac-b1eb-d9b55205353d
      networkId: '1338481'
      udi: PID:C9200L-24P-4G SN:JAE25220R2K
    organizationId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  sample: >
    [
      {
        "status": "string",
        "importId": "string",
        "message": "string"
      }
    ]
"""



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on containerd's platforms Go module
# (https://github.com/containerd/containerd/tree/main/platforms)
#
# Copyright (c) 2023 Felix Fontein <felix@fontein.de>
# Copyright The containerd Authors
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import re


_VALID_STR = re.compile('^[A-Za-z0-9_-]+$')


def _validate_part(string, part, part_name):
    if not part:
        raise ValueError('Invalid platform string "{string}": {part} is empty'.format(string=string, part=part_name))
    if not _VALID_STR.match(part):
        raise ValueError('Invalid platform string "{string}": {part} has invalid characters'.format(string=string, part=part_name))
    return part


# See https://github.com/containerd/containerd/blob/main/platforms/database.go#L32-L38
_KNOWN_OS = (
    "aix", "android", "darwin", "dragonfly", "freebsd", "hurd", "illumos", "ios", "js",
    "linux", "nacl", "netbsd", "openbsd", "plan9", "solaris", "windows", "zos",
)

# See https://github.com/containerd/containerd/blob/main/platforms/database.go#L54-L60
_KNOWN_ARCH = (
    "386", "amd64", "amd64p32", "arm", "armbe", "arm64", "arm64be", "ppc64", "ppc64le",
    "loong64", "mips", "mipsle", "mips64", "mips64le", "mips64p32", "mips64p32le",
    "ppc", "riscv", "riscv64", "s390", "s390x", "sparc", "sparc64", "wasm",
)


def _normalize_os(os_str):
    # See normalizeOS() in https://github.com/containerd/containerd/blob/main/platforms/database.go
    os_str = os_str.lower()
    if os_str == 'macos':
        os_str = 'darwin'
    return os_str


_NORMALIZE_ARCH = {
    ("i386", None): ("386", ""),
    ("x86_64", "v1"): ("amd64", ""),
    ("x86-64", "v1"): ("amd64", ""),
    ("amd64", "v1"): ("amd64", ""),
    ("x86_64", None): ("amd64", None),
    ("x86-64", None): ("amd64", None),
    ("amd64", None): ("amd64", None),
    ("aarch64", "8"): ("arm64", ""),
    ("arm64", "8"): ("arm64", ""),
    ("aarch64", "v8"): ("arm64", ""),
    ("arm64", "v8"): ("arm64", ""),
    ("aarch64", None): ("arm64", None),
    ("arm64", None): ("arm64", None),
    ("armhf", None): ("arm", "v7"),
    ("armel", None): ("arm", "v6"),
    ("arm", ""): ("arm", "v7"),
    ("arm", "5"): ("arm", "v5"),
    ("arm", "6"): ("arm", "v6"),
    ("arm", "7"): ("arm", "v7"),
    ("arm", "8"): ("arm", "v8"),
    ("arm", None): ("arm", None),
}


def _normalize_arch(arch_str, variant_str):
    # See normalizeArch() in https://github.com/containerd/containerd/blob/main/platforms/database.go
    arch_str = arch_str.lower()
    variant_str = variant_str.lower()
    res = _NORMALIZE_ARCH.get((arch_str, variant_str))
    if res is None:
        res = _NORMALIZE_ARCH.get((arch_str, None))
    if res is None:
        return arch_str, variant_str
    if res is not None:
        arch_str = res[0]
        if res[1] is not None:
            variant_str = res[1]
        return arch_str, variant_str


class _Platform(object):
    def __init__(self, os=None, arch=None, variant=None):
        self.os = os
        self.arch = arch
        self.variant = variant
        if variant is not None:
            if arch is None:
                raise ValueError('If variant is given, architecture must be given too')
            if os is None:
                raise ValueError('If variant is given, os must be given too')

    @classmethod
    def parse_platform_string(cls, string, daemon_os=None, daemon_arch=None):
        # See Parse() in https://github.com/containerd/containerd/blob/main/platforms/platforms.go
        if string is None:
            return cls()
        if not string:
            raise ValueError('Platform string must be non-empty')
        parts = string.split('/', 2)
        arch = None
        variant = None
        if len(parts) == 1:
            _validate_part(string, string, 'OS/architecture')
            # The part is either OS or architecture
            os = _normalize_os(string)
            if os in _KNOWN_OS:
                if daemon_arch is not None:
                    arch, variant = _normalize_arch(daemon_arch, '')
                return cls(os=os, arch=arch, variant=variant)
            arch, variant = _normalize_arch(os, '')
            if arch in _KNOWN_ARCH:
                return cls(
                    os=_normalize_os(daemon_os) if daemon_os else None,
                    arch=arch or None,
                    variant=variant or None,
                )
            raise ValueError('Invalid platform string "{0}": unknown OS or architecture'.format(string))
        os = _validate_part(string, parts[0], 'OS')
        if not os:
            raise ValueError('Invalid platform string "{0}": OS is empty'.format(string))
        arch = _validate_part(string, parts[1], 'architecture') if len(parts) > 1 else None
        if arch is not None and not arch:
            raise ValueError('Invalid platform string "{0}": architecture is empty'.format(string))
        variant = _validate_part(string, parts[2], 'variant') if len(parts) > 2 else None
        if variant is not None and not variant:
            raise ValueError('Invalid platform string "{0}": variant is empty'.format(string))
        arch, variant = _normalize_arch(arch, variant or '')
        if len(parts) == 2 and arch == 'arm' and variant == 'v7':
            variant = None
        if len(parts) == 3 and arch == 'arm64' and variant == '':
            variant = 'v8'
        return cls(os=_normalize_os(os), arch=arch, variant=variant or None)

    def __str__(self):
        if self.variant:
            parts = [self.os, self.arch, self.variant]
        elif self.os:
            if self.arch:
                parts = [self.os, self.arch]
            else:
                parts = [self.os]
        elif self.arch is not None:
            parts = [self.arch]
        else:
            parts = []
        return '/'.join(parts)

    def __repr__(self):
        return '_Platform(os={os!r}, arch={arch!r}, variant={variant!r})'.format(os=self.os, arch=self.arch, variant=self.variant)

    def __eq__(self, other):
        return self.os == other.os and self.arch == other.arch and self.variant == other.variant


def normalize_platform_string(string, daemon_os=None, daemon_arch=None):
    return str(_Platform.parse_platform_string(string, daemon_os=daemon_os, daemon_arch=daemon_arch))


def compose_platform_string(os=None, arch=None, variant=None, daemon_os=None, daemon_arch=None):
    if os is None and daemon_os is not None:
        os = _normalize_os(daemon_os)
    if arch is None and daemon_arch is not None:
        arch, variant = _normalize_arch(daemon_arch, variant or '')
        variant = variant or None
    return str(_Platform(os=os, arch=arch, variant=variant or None))


def compare_platform_strings(string1, string2):
    return _Platform.parse_platform_string(string1) == _Platform.parse_platform_string(string2)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_traffic_analysis
short_description: Resource module for networks _trafficanalysis
description:
- Manage operation update of the resource networks _trafficanalysis.
- Update the traffic analysis settings for a network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  customPieChartItems:
    description: The list of items that make up the custom pie chart for traffic reporting.
    elements: dict
    suboptions:
      name:
        description: The name of the custom pie chart item.
        type: str
      type:
        description: The signature type for the custom pie chart item. Can be one of
          'host', 'port' or 'ipRange'.
        type: str
      value:
        description: The value of the custom pie chart item. Valid syntax depends on
          the signature type of the chart item (see sample request/response for more
          details).
        type: str
    type: list
  mode:
    description: The traffic analysis mode for the network. Can be one of 'disabled'
      (do not collect traffic types), 'basic' (collect generic traffic categories),
      or 'detailed' (collect destination hostnames).
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks updateNetworkTrafficAnalysis
  description: Complete reference of the updateNetworkTrafficAnalysis API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-traffic-analysis
notes:
  - SDK Method used are
    networks.Networks.update_network_traffic_analysis,

  - Paths used are
    put /networks/{networkId}/trafficAnalysis,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_traffic_analysis:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    customPieChartItems:
    - name: Item from hostname
      type: host
      value: example.com
    - name: Item from port
      type: port
      value: '440'
    - name: Item from IP
      type: ipRange
      value: 192.1.0.0
    - name: Item from IP range (CIDR)
      type: ipRange
      value: 192.2.0.0/16
    - name: Item from IP range with port
      type: ipRange
      value: 192.3.0.0/16:80
    mode: detailed
    networkId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The arg spec for the asa_acls module
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


class AclsArgs(object):
    """The arg spec for the asa_acls module"""

    def __init__(self, **kwargs):
        pass

    argument_spec = {
        "config": {
            "type": "dict",
            "options": {
                "acls": {
                    "elements": "dict",
                    "type": "list",
                    "options": {
                        "name": {"required": True, "type": "str"},
                        "acl_type": {
                            "choices": ["extended", "standard"],
                            "type": "str",
                        },
                        "rename": {"type": "str"},
                        "aces": {
                            "elements": "dict",
                            "type": "list",
                            "options": {
                                "grant": {
                                    "choices": ["permit", "deny"],
                                    "type": "str",
                                },
                                "line": {"type": "int"},
                                "remark": {"type": "str"},
                                "source": {
                                    "type": "dict",
                                    "options": {
                                        "address": {"type": "str"},
                                        "netmask": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "any4": {"type": "bool"},
                                        "any6": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "interface": {"type": "str"},
                                        "object_group": {"type": "str"},
                                        "port_protocol": {
                                            "type": "dict",
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "start": {
                                                            "type": "int",
                                                        },
                                                        "end": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "destination": {
                                    "type": "dict",
                                    "options": {
                                        "address": {"type": "str"},
                                        "netmask": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "any4": {"type": "bool"},
                                        "any6": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "interface": {"type": "str"},
                                        "object_group": {"type": "str"},
                                        "service_object_group": {
                                            "type": "str",
                                        },
                                        "port_protocol": {
                                            "type": "dict",
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "start": {
                                                            "type": "int",
                                                        },
                                                        "end": {"type": "int"},
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "protocol": {"type": "str"},
                                "protocol_options": {
                                    "type": "dict",
                                    "options": {
                                        "protocol_number": {"type": "int"},
                                        "ahp": {"type": "bool"},
                                        "eigrp": {"type": "bool"},
                                        "esp": {"type": "bool"},
                                        "gre": {"type": "bool"},
                                        "icmp": {
                                            "type": "dict",
                                            "options": {
                                                "alternate_address": {
                                                    "type": "bool",
                                                },
                                                "conversion_error": {
                                                    "type": "bool",
                                                },
                                                "echo": {"type": "bool"},
                                                "echo_reply": {"type": "bool"},
                                                "information_reply": {
                                                    "type": "bool",
                                                },
                                                "information_request": {
                                                    "type": "bool",
                                                },
                                                "mask_reply": {"type": "bool"},
                                                "mask_request": {
                                                    "type": "bool",
                                                },
                                                "mobile_redirect": {
                                                    "type": "bool",
                                                },
                                                "parameter_problem": {
                                                    "type": "bool",
                                                },
                                                "redirect": {"type": "bool"},
                                                "router_advertisement": {
                                                    "type": "bool",
                                                },
                                                "router_solicitation": {
                                                    "type": "bool",
                                                },
                                                "source_quench": {
                                                    "type": "bool",
                                                },
                                                "source_route_failed": {
                                                    "type": "bool",
                                                },
                                                "time_exceeded": {
                                                    "type": "bool",
                                                },
                                                "timestamp_reply": {
                                                    "type": "bool",
                                                },
                                                "timestamp_request": {
                                                    "type": "bool",
                                                },
                                                "traceroute": {"type": "bool"},
                                                "unreachable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "icmp6": {
                                            "type": "dict",
                                            "options": {
                                                "echo": {"type": "bool"},
                                                "echo_reply": {"type": "bool"},
                                                "membership_query": {
                                                    "type": "bool",
                                                },
                                                "membership_reduction": {
                                                    "type": "bool",
                                                },
                                                "membership_report": {
                                                    "type": "bool",
                                                },
                                                "neighbor_advertisement": {
                                                    "type": "bool",
                                                },
                                                "neighbor_redirect": {
                                                    "type": "bool",
                                                },
                                                "neighbor_solicitation": {
                                                    "type": "bool",
                                                },
                                                "packet_too_big": {
                                                    "type": "bool",
                                                },
                                                "parameter_problem": {
                                                    "type": "bool",
                                                },
                                                "router_advertisement": {
                                                    "type": "bool",
                                                },
                                                "router_renumbering": {
                                                    "type": "bool",
                                                },
                                                "router_solicitation": {
                                                    "type": "bool",
                                                },
                                                "time_exceeded": {
                                                    "type": "bool",
                                                },
                                                "unreachable": {
                                                    "type": "bool",
                                                },
                                            },
                                        },
                                        "igmp": {"type": "bool"},
                                        "igrp": {"type": "bool"},
                                        "ip": {"type": "bool"},
                                        "ipinip": {"type": "bool"},
                                        "ipsec": {"type": "bool"},
                                        "nos": {"type": "bool"},
                                        "ospf": {"type": "bool"},
                                        "pcp": {"type": "bool"},
                                        "pim": {"type": "bool"},
                                        "pptp": {"type": "bool"},
                                        "sctp": {"type": "bool"},
                                        "snp": {"type": "bool"},
                                        "tcp": {"type": "bool"},
                                        "udp": {"type": "bool"},
                                    },
                                },
                                "inactive": {"type": "bool"},
                                "log": {
                                    "type": "str",
                                    "choices": [
                                        "default",
                                        "alerts",
                                        "critical",
                                        "debugging",
                                        "disable",
                                        "emergencies",
                                        "errors",
                                        "informational",
                                        "interval",
                                        "notifications",
                                        "warnings",
                                    ],
                                },
                                "time_range": {"type": "str"},
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }



# -*- coding: utf-8 -*-
# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

""" Resolves OpenSSL issues in some servers:
      https://lukasa.co.uk/2013/01/Choosing_SSL_Version_In_Requests/
      https://github.com/kennethreitz/requests/pull/799
"""

from ansible_collections.community.docker.plugins.module_utils.version import StrictVersion

from .._import_helper import HTTPAdapter, urllib3
from .basehttpadapter import BaseHTTPAdapter


PoolManager = urllib3.poolmanager.PoolManager


class SSLHTTPAdapter(BaseHTTPAdapter):
    '''An HTTPS Transport Adapter that uses an arbitrary SSL version.'''

    __attrs__ = HTTPAdapter.__attrs__ + ['assert_hostname', 'ssl_version']

    def __init__(self, ssl_version=None, assert_hostname=None, **kwargs):
        self.ssl_version = ssl_version
        self.assert_hostname = assert_hostname
        super(SSLHTTPAdapter, self).__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        kwargs = {
            'num_pools': connections,
            'maxsize': maxsize,
            'block': block,
        }
        if self.assert_hostname is not None:
            kwargs['assert_hostname'] = self.assert_hostname
        if self.ssl_version and self.can_override_ssl_version():
            kwargs['ssl_version'] = self.ssl_version

        self.poolmanager = PoolManager(**kwargs)

    def get_connection(self, *args, **kwargs):
        """
        Ensure assert_hostname is set correctly on our pool

        We already take care of a normal poolmanager via init_poolmanager

        But we still need to take care of when there is a proxy poolmanager
        """
        conn = super(SSLHTTPAdapter, self).get_connection(*args, **kwargs)
        if self.assert_hostname is not None and conn.assert_hostname != self.assert_hostname:
            conn.assert_hostname = self.assert_hostname
        return conn

    def can_override_ssl_version(self):
        urllib_ver = urllib3.__version__.split('-')[0]
        if urllib_ver is None:
            return False
        if urllib_ver == 'dev':
            return True
        return StrictVersion(urllib_ver) > StrictVersion('1.5')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import abc

from ansible.module_utils import six
from ansible.module_utils.common.text.converters import to_native
from ansible.module_utils.six import PY3
from ansible.module_utils.urls import fetch_url, open_url, NoSSLError, ConnectionError
import ansible.module_utils.six.moves.urllib.error as urllib_error


class NetworkError(Exception):
    pass


@six.add_metaclass(abc.ABCMeta)
class HTTPHelper(object):
    @abc.abstractmethod
    def fetch_url(self, url, method='GET', headers=None, data=None, timeout=None):
        """
        Execute a HTTP request and return a tuple (response_content, info).

        In case of errors, either raise NetworkError or terminate the program (for modules only!).
        """


class ModuleHTTPHelper(HTTPHelper):
    def __init__(self, module):
        self.module = module

    def fetch_url(self, url, method='GET', headers=None, data=None, timeout=None):
        response, info = fetch_url(self.module, url, method=method, headers=headers, data=data, timeout=timeout)
        try:
            # In Python 2, reading from a closed response yields a TypeError.
            # In Python 3, read() simply returns ''
            if PY3 and response.closed:
                raise TypeError
            content = response.read()
        except (AttributeError, TypeError):
            content = info.pop('body', None)
        return content, info


class OpenURLHelper(HTTPHelper):
    def fetch_url(self, url, method='GET', headers=None, data=None, timeout=None):
        info = {}
        try:
            req = open_url(url, method=method, headers=headers, data=data, timeout=timeout)
            result = req.read()
            info.update(dict((k.lower(), v) for k, v in req.info().items()))
            info['status'] = req.code
            info['url'] = req.geturl()
            req.close()
        except urllib_error.HTTPError as e:
            try:
                result = e.read()
            except AttributeError:
                result = ''
            try:
                info.update(dict((k.lower(), v) for k, v in e.info().items()))
            except Exception:
                pass
            info['status'] = e.code
        except NoSSLError as e:
            raise NetworkError('Cannot connect via SSL: {0}'.format(to_native(e)))
        except (ConnectionError, ValueError) as e:
            raise NetworkError('Connection error: {0}'.format(to_native(e)))

        return result, info



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: file_import
short_description: Resource module for File Import
description:
- Manage operation create of the resource File Import.
- Uploads a new file within a specific nameSpace.
version_added: '6.0.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  filePath:
    description: File absolute path.
    type: str
  nameSpace:
    description: NameSpace path parameter.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for File UploadFile
  description: Complete reference of the UploadFile API.
  link: https://developer.cisco.com/docs/dna-center/#!upload-file
notes:
  - SDK Method used are
    file.File.upload_file,

  - Paths used are
    post /dna/intent/api/v1/file/{nameSpace},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.file_import:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    filePath: /tmp/uploads/Test-242.bin
    nameSpace: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020, Doug Stanley <doug+ansible@technologixllc.com>
# Copyright (c) 2021, Andrew Pantuso (@ajpantuso) <ajpantuso@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import os
import re
from contextlib import contextmanager
from struct import Struct

from ansible.module_utils.six import PY3

# Protocol References
# -------------------
# https://datatracker.ietf.org/doc/html/rfc4251
# https://datatracker.ietf.org/doc/html/rfc4253
# https://datatracker.ietf.org/doc/html/rfc5656
# https://datatracker.ietf.org/doc/html/rfc8032
#
# Inspired by:
# ------------
# https://github.com/pyca/cryptography/blob/main/src/cryptography/hazmat/primitives/serialization/ssh.py
# https://github.com/paramiko/paramiko/blob/master/paramiko/message.py

if PY3:
    long = int

# 0 (False) or 1 (True) encoded as a single byte
_BOOLEAN = Struct(b'?')
# Unsigned 8-bit integer in network-byte-order
_UBYTE = Struct(b'!B')
_UBYTE_MAX = 0xFF
# Unsigned 32-bit integer in network-byte-order
_UINT32 = Struct(b'!I')
# Unsigned 32-bit little endian integer
_UINT32_LE = Struct(b'<I')
_UINT32_MAX = 0xFFFFFFFF
# Unsigned 64-bit integer in network-byte-order
_UINT64 = Struct(b'!Q')
_UINT64_MAX = 0xFFFFFFFFFFFFFFFF


def any_in(sequence, *elements):
    return any(e in sequence for e in elements)


def file_mode(path):
    if not os.path.exists(path):
        return 0o000
    return os.stat(path).st_mode & 0o777


def parse_openssh_version(version_string):
    """Parse the version output of ssh -V and return version numbers that can be compared"""

    parsed_result = re.match(
        r"^.*openssh_(?P<version>[0-9.]+)(p?[0-9]+)[^0-9]*.*$", version_string.lower()
    )
    if parsed_result is not None:
        version = parsed_result.group("version").strip()
    else:
        version = None

    return version


@contextmanager
def secure_open(path, mode):
    fd = os.open(path, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, mode)
    try:
        yield fd
    finally:
        os.close(fd)


def secure_write(path, mode, content):
    with secure_open(path, mode) as fd:
        os.write(fd, content)


# See https://datatracker.ietf.org/doc/html/rfc4251#section-5 for SSH data types
class OpensshParser(object):
    """Parser for OpenSSH encoded objects"""
    BOOLEAN_OFFSET = 1
    UINT32_OFFSET = 4
    UINT64_OFFSET = 8

    def __init__(self, data):
        if not isinstance(data, (bytes, bytearray)):
            raise TypeError("Data must be bytes-like not %s" % type(data))

        self._data = memoryview(data) if PY3 else data
        self._pos = 0

    def boolean(self):
        next_pos = self._check_position(self.BOOLEAN_OFFSET)

        value = _BOOLEAN.unpack(self._data[self._pos:next_pos])[0]
        self._pos = next_pos
        return value

    def uint32(self):
        next_pos = self._check_position(self.UINT32_OFFSET)

        value = _UINT32.unpack(self._data[self._pos:next_pos])[0]
        self._pos = next_pos
        return value

    def uint64(self):
        next_pos = self._check_position(self.UINT64_OFFSET)

        value = _UINT64.unpack(self._data[self._pos:next_pos])[0]
        self._pos = next_pos
        return value

    def string(self):
        length = self.uint32()

        next_pos = self._check_position(length)

        value = self._data[self._pos:next_pos]
        self._pos = next_pos
        # Cast to bytes is required as a memoryview slice is itself a memoryview
        return value if not PY3 else bytes(value)

    def mpint(self):
        return self._big_int(self.string(), "big", signed=True)

    def name_list(self):
        raw_string = self.string()
        return raw_string.decode('ASCII').split(',')

    # Convenience function, but not an official data type from SSH
    def string_list(self):
        result = []
        raw_string = self.string()

        if raw_string:
            parser = OpensshParser(raw_string)
            while parser.remaining_bytes():
                result.append(parser.string())

        return result

    # Convenience function, but not an official data type from SSH
    def option_list(self):
        result = []
        raw_string = self.string()

        if raw_string:
            parser = OpensshParser(raw_string)

            while parser.remaining_bytes():
                name = parser.string()
                data = parser.string()
                if data:
                    # data is doubly-encoded
                    data = OpensshParser(data).string()
                result.append((name, data))

        return result

    def seek(self, offset):
        self._pos = self._check_position(offset)

        return self._pos

    def remaining_bytes(self):
        return len(self._data) - self._pos

    def _check_position(self, offset):
        if self._pos + offset > len(self._data):
            raise ValueError("Insufficient data remaining at position: %s" % self._pos)
        elif self._pos + offset < 0:
            raise ValueError("Position cannot be less than zero.")
        else:
            return self._pos + offset

    @classmethod
    def signature_data(cls, signature_string):
        signature_data = {}

        parser = cls(signature_string)
        signature_type = parser.string()
        signature_blob = parser.string()

        blob_parser = cls(signature_blob)
        if signature_type in (b'ssh-rsa', b'rsa-sha2-256', b'rsa-sha2-512'):
            # https://datatracker.ietf.org/doc/html/rfc4253#section-6.6
            # https://datatracker.ietf.org/doc/html/rfc8332#section-3
            signature_data['s'] = cls._big_int(signature_blob, "big")
        elif signature_type == b'ssh-dss':
            # https://datatracker.ietf.org/doc/html/rfc4253#section-6.6
            signature_data['r'] = cls._big_int(signature_blob[:20], "big")
            signature_data['s'] = cls._big_int(signature_blob[20:], "big")
        elif signature_type in (b'ecdsa-sha2-nistp256', b'ecdsa-sha2-nistp384', b'ecdsa-sha2-nistp521'):
            # https://datatracker.ietf.org/doc/html/rfc5656#section-3.1.2
            signature_data['r'] = blob_parser.mpint()
            signature_data['s'] = blob_parser.mpint()
        elif signature_type == b'ssh-ed25519':
            # https://datatracker.ietf.org/doc/html/rfc8032#section-5.1.2
            signature_data['R'] = cls._big_int(signature_blob[:32], "little")
            signature_data['S'] = cls._big_int(signature_blob[32:], "little")
        else:
            raise ValueError("%s is not a valid signature type" % signature_type)

        signature_data['signature_type'] = signature_type

        return signature_data

    @classmethod
    def _big_int(cls, raw_string, byte_order, signed=False):
        if byte_order not in ("big", "little"):
            raise ValueError("Byte_order must be one of (big, little) not %s" % byte_order)

        if PY3:
            return int.from_bytes(raw_string, byte_order, signed=signed)

        result = 0
        byte_length = len(raw_string)

        if byte_length > 0:
            # Check sign-bit
            msb = raw_string[0] if byte_order == "big" else raw_string[-1]
            negative = bool(ord(msb) & 0x80)
            # Match pad value for two's complement
            pad = b'\xFF' if signed and negative else b'\x00'
            # The definition of ``mpint`` enforces that unnecessary bytes are not encoded so they are added back
            pad_length = (4 - byte_length % 4)
            if pad_length < 4:
                raw_string = pad * pad_length + raw_string if byte_order == "big" else raw_string + pad * pad_length
                byte_length += pad_length
            # Accumulate arbitrary precision integer bytes in the appropriate order
            if byte_order == "big":
                for i in range(0, byte_length, cls.UINT32_OFFSET):
                    left_shift = result << cls.UINT32_OFFSET * 8
                    result = left_shift + _UINT32.unpack(raw_string[i:i + cls.UINT32_OFFSET])[0]
            else:
                for i in range(byte_length, 0, -cls.UINT32_OFFSET):
                    left_shift = result << cls.UINT32_OFFSET * 8
                    result = left_shift + _UINT32_LE.unpack(raw_string[i - cls.UINT32_OFFSET:i])[0]
            # Adjust for two's complement
            if signed and negative:
                result -= 1 << (8 * byte_length)

        return result


class _OpensshWriter(object):
    """Writes SSH encoded values to a bytes-like buffer

    .. warning::
        This class is a private API and must not be exported outside of the openssh module_utils.
        It is not to be used to construct Openssh objects, but rather as a utility to assist
        in validating parsed material.
    """
    def __init__(self, buffer=None):
        if buffer is not None:
            if not isinstance(buffer, (bytes, bytearray)):
                raise TypeError("Buffer must be a bytes-like object not %s" % type(buffer))
        else:
            buffer = bytearray()

        self._buff = buffer

    def boolean(self, value):
        if not isinstance(value, bool):
            raise TypeError("Value must be of type bool not %s" % type(value))

        self._buff.extend(_BOOLEAN.pack(value))

        return self

    def uint32(self, value):
        if not isinstance(value, int):
            raise TypeError("Value must be of type int not %s" % type(value))
        if value < 0 or value > _UINT32_MAX:
            raise ValueError("Value must be a positive integer less than %s" % _UINT32_MAX)

        self._buff.extend(_UINT32.pack(value))

        return self

    def uint64(self, value):
        if not isinstance(value, (long, int)):
            raise TypeError("Value must be of type (long, int) not %s" % type(value))
        if value < 0 or value > _UINT64_MAX:
            raise ValueError("Value must be a positive integer less than %s" % _UINT64_MAX)

        self._buff.extend(_UINT64.pack(value))

        return self

    def string(self, value):
        if not isinstance(value, (bytes, bytearray)):
            raise TypeError("Value must be bytes-like not %s" % type(value))
        self.uint32(len(value))
        self._buff.extend(value)

        return self

    def mpint(self, value):
        if not isinstance(value, (int, long)):
            raise TypeError("Value must be of type (long, int) not %s" % type(value))

        self.string(self._int_to_mpint(value))

        return self

    def name_list(self, value):
        if not isinstance(value, list):
            raise TypeError("Value must be a list of byte strings not %s" % type(value))

        try:
            self.string(','.join(value).encode('ASCII'))
        except UnicodeEncodeError as e:
            raise ValueError("Name-list's must consist of US-ASCII characters: %s" % e)

        return self

    def string_list(self, value):
        if not isinstance(value, list):
            raise TypeError("Value must be a list of byte string not %s" % type(value))

        writer = _OpensshWriter()
        for s in value:
            writer.string(s)

        self.string(writer.bytes())

        return self

    def option_list(self, value):
        if not isinstance(value, list) or (value and not isinstance(value[0], tuple)):
            raise TypeError("Value must be a list of tuples")

        writer = _OpensshWriter()
        for name, data in value:
            writer.string(name)
            # SSH option data is encoded twice though this behavior is not documented
            writer.string(_OpensshWriter().string(data).bytes() if data else bytes())

        self.string(writer.bytes())

        return self

    @staticmethod
    def _int_to_mpint(num):
        if PY3:
            byte_length = (num.bit_length() + 7) // 8
            try:
                result = num.to_bytes(byte_length, "big", signed=True)
            # Handles values which require \x00 or \xFF to pad sign-bit
            except OverflowError:
                result = num.to_bytes(byte_length + 1, "big", signed=True)
        else:
            result = bytes()
            # 0 and -1 are treated as special cases since they are used as sentinels for all other values
            if num == 0:
                result += b'\x00'
            elif num == -1:
                result += b'\xFF'
            elif num > 0:
                while num >> 32:
                    result = _UINT32.pack(num & _UINT32_MAX) + result
                    num = num >> 32
                # Pack last 4 bytes individually to discard insignificant bytes
                while num:
                    result = _UBYTE.pack(num & _UBYTE_MAX) + result
                    num = num >> 8
                # Zero pad final byte if most-significant bit is 1 as per mpint definition
                if ord(result[0]) & 0x80:
                    result = b'\x00' + result
            else:
                while (num >> 32) < -1:
                    result = _UINT32.pack(num & _UINT32_MAX) + result
                    num = num >> 32
                while num < -1:
                    result = _UBYTE.pack(num & _UBYTE_MAX) + result
                    num = num >> 8
                if not ord(result[0]) & 0x80:
                    result = b'\xFF' + result

        return result

    def bytes(self):
        return bytes(self._buff)



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: docker_prune

short_description: Allows to prune various docker objects

description:
  - Allows to run C(docker container prune), C(docker image prune), C(docker network prune)
    and C(docker volume prune) via the Docker API.

extends_documentation_fragment:
  - community.docker.docker.api_documentation
  - community.docker.attributes
  - community.docker.attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  containers:
    description:
      - Whether to prune containers.
    type: bool
    default: false
  containers_filters:
    description:
      - A dictionary of filter values used for selecting containers to delete.
      - "For example, C(until: 24h)."
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/container_prune/#filtering)
        for more information on possible filters.
    type: dict
  images:
    description:
      - Whether to prune images.
    type: bool
    default: false
  images_filters:
    description:
      - A dictionary of filter values used for selecting images to delete.
      - "For example, C(dangling: true)."
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/image_prune/#filtering)
        for more information on possible filters.
    type: dict
  networks:
    description:
      - Whether to prune networks.
    type: bool
    default: false
  networks_filters:
    description:
      - A dictionary of filter values used for selecting networks to delete.
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/network_prune/#filtering)
        for more information on possible filters.
    type: dict
  volumes:
    description:
      - Whether to prune volumes.
    type: bool
    default: false
  volumes_filters:
    description:
      - A dictionary of filter values used for selecting volumes to delete.
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/volume_prune/#filtering)
        for more information on possible filters.
    type: dict
  builder_cache:
    description:
      - Whether to prune the builder cache.
    type: bool
    default: false

author:
  - "Felix Fontein (@felixfontein)"

notes:
  - The module always returned C(changed=false) before community.docker 3.5.1.

requirements:
  - "Docker API >= 1.25"
'''

EXAMPLES = '''
- name: Prune containers older than 24h
  community.docker.docker_prune:
    containers: true
    containers_filters:
      # only consider containers created more than 24 hours ago
      until: 24h

- name: Prune everything
  community.docker.docker_prune:
    containers: true
    images: true
    networks: true
    volumes: true
    builder_cache: true

- name: Prune everything (including non-dangling images)
  community.docker.docker_prune:
    containers: true
    images: true
    images_filters:
      dangling: false
    networks: true
    volumes: true
    builder_cache: true
'''

RETURN = '''
# containers
containers:
    description:
      - List of IDs of deleted containers.
    returned: O(containers=true)
    type: list
    elements: str
    sample: []
containers_space_reclaimed:
    description:
      - Amount of reclaimed disk space from container pruning in bytes.
    returned: O(containers=true)
    type: int
    sample: 0

# images
images:
    description:
      - List of IDs of deleted images.
    returned: O(images=true)
    type: list
    elements: str
    sample: []
images_space_reclaimed:
    description:
      - Amount of reclaimed disk space from image pruning in bytes.
    returned: O(images=true)
    type: int
    sample: 0

# networks
networks:
    description:
      - List of IDs of deleted networks.
    returned: O(networks=true)
    type: list
    elements: str
    sample: []

# volumes
volumes:
    description:
      - List of IDs of deleted volumes.
    returned: O(volumes=true)
    type: list
    elements: str
    sample: []
volumes_space_reclaimed:
    description:
      - Amount of reclaimed disk space from volumes pruning in bytes.
    returned: O(volumes=true)
    type: int
    sample: 0

# builder_cache
builder_cache_space_reclaimed:
    description:
      - Amount of reclaimed disk space from builder cache pruning in bytes.
    returned: O(builder_cache=true)
    type: int
    sample: 0
'''

import traceback

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.docker.plugins.module_utils.common_api import (
    AnsibleDockerClient,
    RequestException,
)

from ansible_collections.community.docker.plugins.module_utils.util import clean_dict_booleans_for_docker_api

from ansible_collections.community.docker.plugins.module_utils._api.errors import DockerException
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import convert_filters


def main():
    argument_spec = dict(
        containers=dict(type='bool', default=False),
        containers_filters=dict(type='dict'),
        images=dict(type='bool', default=False),
        images_filters=dict(type='dict'),
        networks=dict(type='bool', default=False),
        networks_filters=dict(type='dict'),
        volumes=dict(type='bool', default=False),
        volumes_filters=dict(type='dict'),
        builder_cache=dict(type='bool', default=False),
    )

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        # supports_check_mode=True,
    )

    try:
        result = dict()
        changed = False

        if client.module.params['containers']:
            filters = clean_dict_booleans_for_docker_api(client.module.params.get('containers_filters'))
            params = {'filters': convert_filters(filters)}
            res = client.post_to_json('/containers/prune', params=params)
            result['containers'] = res.get('ContainersDeleted') or []
            result['containers_space_reclaimed'] = res['SpaceReclaimed']
            if result['containers'] or result['containers_space_reclaimed']:
                changed = True

        if client.module.params['images']:
            filters = clean_dict_booleans_for_docker_api(client.module.params.get('images_filters'))
            params = {'filters': convert_filters(filters)}
            res = client.post_to_json('/images/prune', params=params)
            result['images'] = res.get('ImagesDeleted') or []
            result['images_space_reclaimed'] = res['SpaceReclaimed']
            if result['images'] or result['images_space_reclaimed']:
                changed = True

        if client.module.params['networks']:
            filters = clean_dict_booleans_for_docker_api(client.module.params.get('networks_filters'))
            params = {'filters': convert_filters(filters)}
            res = client.post_to_json('/networks/prune', params=params)
            result['networks'] = res.get('NetworksDeleted') or []
            if result['networks']:
                changed = True

        if client.module.params['volumes']:
            filters = clean_dict_booleans_for_docker_api(client.module.params.get('volumes_filters'))
            params = {'filters': convert_filters(filters)}
            res = client.post_to_json('/volumes/prune', params=params)
            result['volumes'] = res.get('VolumesDeleted') or []
            result['volumes_space_reclaimed'] = res['SpaceReclaimed']
            if result['volumes'] or result['volumes_space_reclaimed']:
                changed = True

        if client.module.params['builder_cache']:
            res = client.post_to_json('/build/prune')
            result['builder_cache_space_reclaimed'] = res['SpaceReclaimed']
            if result['builder_cache_space_reclaimed']:
                changed = True

        result['changed'] = changed
        client.module.exit_json(**result)
    except DockerException as e:
        client.fail('An unexpected Docker error occurred: {0}'.format(to_native(e)), exception=traceback.format_exc())
    except RequestException as e:
        client.fail(
            'An unexpected requests error occurred when trying to talk to the Docker daemon: {0}'.format(to_native(e)),
            exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tag_info
short_description: Information module for Tag
description:
- Get all Tag.
- Get Tag by id.
- Returns tag specified by Id.
- Returns the tags for given filter criteria.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  name:
    description:
    - Name query parameter. Tag name is mandatory when filter operation is used.
    type: str
  additionalInfo_nameSpace:
    description:
    - AdditionalInfo.nameSpace query parameter.
    type: str
  additionalInfo_attributes:
    description:
    - AdditionalInfo.attributes query parameter.
    type: str
  level:
    description:
    - Level query parameter.
    type: str
  offset:
    description:
    - Offset query parameter.
    type: int
  limit:
    description:
    - Limit query parameter.
    type: int
  size:
    description:
    - Size query parameter. Size in kilobytes(KB).
    type: str
  field:
    description:
    - >
      Field query parameter. Available field names are
      'name,id,parentId,type,additionalInfo.nameSpace,additionalInfo.attributes'.
    type: str
  sortBy:
    description:
    - SortBy query parameter. Only supported attribute is name. SortyBy is mandatory when order is used.
    type: str
  order:
    description:
    - Order query parameter. Available values are asc and des.
    type: str
  systemTag:
    description:
    - SystemTag query parameter.
    type: str
  id:
    description:
    - Id path parameter. Tag ID.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Tag GetTag
  description: Complete reference of the GetTag API.
  link: https://developer.cisco.com/docs/dna-center/#!get-tag
- name: Cisco DNA Center documentation for Tag GetTagById
  description: Complete reference of the GetTagById API.
  link: https://developer.cisco.com/docs/dna-center/#!get-tag-by-id
notes:
  - SDK Method used are
    tag.Tag.get_tag,
    tag.Tag.get_tag_by_id,

  - Paths used are
    get /dna/intent/api/v1/tag,
    get /dna/intent/api/v1/tag/{id},

"""

EXAMPLES = r"""
- name: Get all Tag
  cisco.dnac.tag_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    name: string
    additionalInfo_nameSpace: string
    additionalInfo_attributes: string
    level: string
    offset: 0
    limit: 0
    size: string
    field: string
    sortBy: string
    order: string
    systemTag: string
  register: result

- name: Get Tag by id
  cisco.dnac.tag_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "systemTag": true,
        "description": "string",
        "dynamicRules": [
          {
            "memberType": "string",
            "rules": {
              "values": [
                "string"
              ],
              "items": "string",
              "operation": "string",
              "name": "string",
              "value": "string"
            }
          }
        ],
        "name": "string",
        "id": "string",
        "instanceTenantId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_firmware_upgrades_staged_events_rollbacks
short_description: Resource module for networks _firmwareupgrades _staged _events _rollbacks
description:
- Manage operation create of the resource networks _firmwareupgrades _staged _events _rollbacks.
- Rollback a Staged Upgrade Event for a network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  reasons:
    description: The reason for rolling back the staged upgrade.
    elements: dict
    suboptions:
      category:
        description: Reason for the rollback.
        type: str
      comment:
        description: Additional comment about the rollback.
        type: str
    type: list
  stages:
    description: All completed or in-progress stages in the network with their new start
      times. All pending stages will be canceled.
    elements: dict
    suboptions:
      group:
        description: The Staged Upgrade Group containing the name and ID.
        suboptions:
          id:
            description: ID of the Staged Upgrade Group.
            type: str
        type: dict
      milestones:
        description: The Staged Upgrade Milestones for the specific stage.
        suboptions:
          scheduledFor:
            description: The start time of the staged upgrade stage. (In ISO-8601 format,
              in the time zone of the network.).
            type: str
        type: dict
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks rollbacksNetworkFirmwareUpgradesStagedEvents
  description: Complete reference of the rollbacksNetworkFirmwareUpgradesStagedEvents API.
  link: https://developer.cisco.com/meraki/api-v1/#!rollbacks-network-firmware-upgrades-staged-events
notes:
  - SDK Method used are
    networks.Networks.rollbacks_network_firmware_upgrades_staged_events,

  - Paths used are
    post /networks/{networkId}/firmwareUpgrades/staged/events/rollbacks,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_firmware_upgrades_staged_events_rollbacks:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    networkId: string
    reasons:
    - category: performance
      comment: Network was slower with the upgrade
    stages:
    - group:
        id: '1234'
      milestones:
        scheduledFor: '2018-02-11T00:00:00Z'

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "products": {
        "switch": {
          "nextUpgrade": {
            "toVersion": {
              "id": "string",
              "shortName": "string"
            }
          }
        }
      },
      "stages": [
        {
          "group": {
            "id": "string",
            "name": "string",
            "description": "string"
          },
          "milestones": {
            "scheduledFor": "string",
            "startedAt": "string",
            "completedAt": "string",
            "canceledAt": "string"
          },
          "status": "string"
        }
      ],
      "reasons": [
        {
          "category": "string",
          "comment": "string"
        }
      ]
    }
"""
